/**
 * @preserve
 * Bundle of packer-cli
 * Version: 2.17.3
 * Description: Full-fledged CLI tool to generate and package library Node modules compliant with Browser and NodeJS. Packer CLI support all modern style, unit test and script transpiler tools.
 * License: MIT
 **/

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var gulp = _interopDefault(require('gulp'));
var chalk = _interopDefault(require('chalk'));
var clean = _interopDefault(require('gulp-clean'));
var path = require('path');
var path__default = _interopDefault(path);
var fs = _interopDefault(require('fs'));
var inspector = require('schema-inspector');
var inspector__default = _interopDefault(inspector);
var forOwn = _interopDefault(require('lodash/forOwn'));
var child_process = require('child_process');
var mergeWith = _interopDefault(require('lodash/mergeWith'));
var chokidar = _interopDefault(require('chokidar'));
var padStart = _interopDefault(require('lodash/padStart'));
var gulpAdd = _interopDefault(require('gulp-add'));
var chmod = _interopDefault(require('gulp-chmod'));
var merge = _interopDefault(require('lodash/merge'));
var through = _interopDefault(require('through2'));
var PluginError = _interopDefault(require('plugin-error'));
var handlebars = _interopDefault(require('handlebars'));
var glob = _interopDefault(require('glob-to-regexp'));
var terser = _interopDefault(require('terser'));
var get = _interopDefault(require('lodash/get'));
var codeFrameColumns = _interopDefault(require('@babel/code-frame'));
var rollup = require('rollup');
var rollupIgnoreImport = _interopDefault(require('rollup-plugin-ignore-import'));
var rollupPostCss = _interopDefault(require('rollup-plugin-postcss'));
var rollupReplace = _interopDefault(require('rollup-plugin-re'));
var rollupIgnore = _interopDefault(require('rollup-plugin-ignore'));
var rollupResolve = _interopDefault(require('rollup-plugin-node-resolve'));
var rollupCommonjs = _interopDefault(require('rollup-plugin-commonjs'));
var rollupTypescript = _interopDefault(require('rollup-plugin-typescript2'));
var rollupBabel = _interopDefault(require('rollup-plugin-babel'));
var rollupHandlebars = _interopDefault(require('rollup-plugin-hbs'));
var rollupImage = _interopDefault(require('rollup-plugin-img'));
var rollupFilesize = _interopDefault(require('rollup-plugin-filesize'));
var rollupGlobals = _interopDefault(require('rollup-plugin-node-globals'));
var rollupJson = _interopDefault(require('rollup-plugin-json'));
var isEmail = _interopDefault(require('validator/lib/isEmail'));
var isUrl = _interopDefault(require('validator/lib/isURL'));
var npmValidate = _interopDefault(require('validate-npm-package-name'));
var inquirer = _interopDefault(require('inquirer'));
var rollupIstanbul = _interopDefault(require('rollup-plugin-istanbul'));
var karmaRollupPreprocessor = _interopDefault(require('karma-rollup-preprocessor'));
var rollupBrowserSync = _interopDefault(require('rollup-plugin-browsersync'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

/**
 * Schema inspector packer config validation schema.
 */

var packerSchema = {
  type: 'object',
  optional: false,
  $mapDef: true,
  properties: {
    entry: {
      type: 'string',
      optional: false,
      def: 'index.js'
    },
    source: {
      type: 'string',
      optional: false,
      def: 'src'
    },
    dist: {
      type: 'string',
      optional: false,
      def: 'dist'
    },
    tmp: {
      type: 'string',
      optional: false,
      def: '.tmp'
    },
    compiler: {
      type: 'object',
      optional: false,
      $mapDef: true,
      properties: {
        dependencyMapMode: {
          type: 'string',
          optional: false,
          def: 'cross-map-peer-dependency',
          pattern: /^(cross-map-peer-dependency|cross-map-dependency|map-dependency|map-peer-dependency|all)$/
        },
        packageFieldsToCopy: {
          type: 'array',
          optional: false,
          def: ['name', 'version', 'description', 'keywords', 'author', 'repository', 'license', 'bugs', 'homepage'],
          items: {
            type: 'string'
          }
        },
        sourceMap: {
          type: ['string', 'boolean'],
          optional: false,
          pattern: /^(inline)$/,
          def: true
        },
        customRollupPluginExtractor: {
          type: ['null', 'function'],
          optional: false,
          def: null
        },
        build: {
          type: 'object',
          optional: false,
          $mapDef: true,
          properties: {
            bundleMin: {
              type: 'boolean',
              optional: false,
              def: true
            },
            es5: {
              type: 'boolean',
              optional: false,
              def: false
            },
            es5Min: {
              type: 'boolean',
              optional: false,
              def: false
            },
            esnext: {
              type: 'boolean',
              optional: false,
              def: true
            },
            esnextMin: {
              type: 'boolean',
              optional: false,
              def: true
            }
          }
        },
        buildMode: {
          type: 'string',
          optional: false,
          def: 'browser',
          pattern: /^(browser|node|node-cli)$/
        },
        script: {
          type: 'object',
          optional: false,
          $mapDef: true,
          properties: {
            preprocessor: {
              type: 'string',
              optional: false,
              def: 'none',
              pattern: /^(none|typescript)$/
            },
            tsd: {
              type: 'string',
              optional: false,
              def: 'typings'
            },
            image: {
              type: 'object',
              optional: false,
              $mapDef: true,
              properties: {
                inlineLimit: {
                  type: 'number',
                  optional: false,
                  gt: 0,
                  def: 1000000
                },
                outDir: {
                  type: 'string',
                  optional: false,
                  def: 'images'
                }
              }
            }
          }
        },
        style: {
          type: ['object', 'boolean'],
          optional: false,
          $acceptOnly: 'false',
          $mapDef: true,
          properties: {
            inline: {
              type: 'boolean',
              optional: false,
              def: false
            },
            outDir: {
              type: 'string',
              optional: false,
              def: 'styles'
            },
            preprocessor: {
              type: 'string',
              optional: false,
              def: 'none',
              pattern: /^(scss|sass|less|stylus|none)$/
            },
            image: {
              type: 'object',
              optional: false,
              $mapDef: true,
              properties: {
                inlineLimit: {
                  type: 'number',
                  optional: false,
                  gt: 0,
                  def: 1000000
                },
                outDir: {
                  type: 'string',
                  optional: false,
                  def: 'images'
                }
              }
            }
          }
        },
        concurrentBuild: {
          type: 'boolean',
          optional: false,
          def: true
        },
        advanced: {
          type: 'object',
          optional: false,
          $mapDef: true,
          properties: {
            rollup: {
              type: 'object',
              optional: false,
              $mapDef: true,
              properties: {
                inputOptions: {
                  type: 'object',
                  optional: false,
                  def: {}
                },
                outputOptions: {
                  type: 'object',
                  optional: false,
                  def: {}
                },
                watchOptions: {
                  type: 'object',
                  optional: false,
                  def: {}
                },
                pluginOptions: {
                  type: 'object',
                  optional: false,
                  $mapDef: true,
                  properties: {
                    ignoreImport: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    postCss: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    nodeResolve: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    commonjs: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    json: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    globals: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    babel: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    typescript: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    replace: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    image: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    handlebars: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    filesize: {
                      type: 'object',
                      optional: false,
                      def: {}
                    },
                    browserSync: {
                      type: 'object',
                      optional: false,
                      def: {}
                    }
                  }
                }
              }
            },
            other: {
              type: 'object',
              optional: false,
              $mapDef: true,
              properties: {
                terser: {
                  type: 'object',
                  optional: false,
                  def: {}
                },
                cssnano: {
                  type: 'object',
                  optional: false,
                  def: {}
                }
              }
            }
          }
        }
      }
    },
    assetPaths: {
      type: 'array',
      optional: false,
      items: {
        type: 'string'
      },
      def: []
    },
    copy: {
      type: 'array',
      optional: false,
      items: {
        type: 'string'
      },
      def: ['README.md', 'LICENSE']
    },
    ignore: {
      type: 'array',
      optional: false,
      items: {
        type: 'string'
      },
      def: []
    },
    replacePatterns: {
      type: 'array',
      optional: false,
      items: {
        type: 'object',
        properties: {
          include: {
            type: ['string', 'array'],
            optional: false,
            items: {
              type: 'string'
            }
          },
          exclude: {
            type: ['string', 'array'],
            optional: true,
            items: {
              type: 'string'
            }
          },
          test: {
            type: ['string', 'object'],
            optional: true
          },
          replace: {
            type: 'string',
            optional: true
          },
          match: {
            type: 'object',
            optional: true
          },
          text: {
            type: 'string',
            optional: true
          },
          file: {
            type: 'string',
            optional: true
          },
          pathReplaceCallback: {
            type: 'function',
            optional: true
          }
        }
      },
      def: []
    },
    bundle: {
      type: 'object',
      optional: false,
      $mapDef: true,
      properties: {
        externals: {
          type: 'array',
          optional: false,
          items: {
            type: 'string'
          },
          def: []
        },
        globals: {
          type: 'object',
          optional: false
        },
        mapExternals: {
          type: 'boolean',
          optional: false,
          def: true
        },
        format: {
          type: 'string',
          optional: false,
          def: 'umd',
          pattern: /^(umd|amd|iife|system|cjs|esm)$/
        },
        namespace: {
          type: 'string',
          optional: false,
          def: 'com.lib'
        },
        amd: {
          type: 'object',
          optional: false,
          $mapDef: true,
          properties: {
            define: {
              type: 'string',
              optional: false,
              def: ''
            },
            id: {
              type: 'string',
              optional: false,
              def: ''
            }
          }
        }
      }
    },
    test: {
      type: 'object',
      optional: false,
      $mapDef: true,
      properties: {
        framework: {
          type: 'string',
          optional: false,
          def: 'jasmine',
          pattern: /^(jasmine|mocha|jest)$/
        },
        environment: {
          type: 'string',
          optional: false,
          def: 'jsdom',
          pattern: /^(browser|jsdom|node|enzyme)$/
        },
        advanced: {
          type: 'object',
          optional: false,
          $mapDef: true,
          properties: {
            jasmine: {
              default: {
                type: 'string',
                optional: false,
                def: 'jasmine --config=jasmine.json'
              },
              watch: {
                type: 'string',
                optional: false,
                def: 'jasmine --config=jasmine.json'
              },
              coverageDefault: {
                type: 'string',
                optional: false,
                def: 'nyc jasmine --config=jasmine.json'
              },
              coverageWatch: {
                type: 'string',
                optional: false,
                def: 'nyc jasmine --config=jasmine.json'
              }
            },
            mocha: {
              default: {
                type: 'string',
                optional: false,
                def: 'mocha --opts mocha.opts "./{,!(node_modules)/**/}*.[sS]pec.<ext-glob>"'
              },
              watch: {
                type: 'string',
                optional: false,
                def: 'mocha --opts mocha.opts --watch "./{,!(node_modules)/**/}*.[sS]pec.<ext-glob>"'
              },
              coverageDefault: {
                type: 'string',
                optional: false,
                def: 'nyc mocha --opts mocha.opts --watch "./{,!(node_modules)/**/}*.[sS]pec.<ext-glob>"'
              },
              coverageWatch: {
                type: 'string',
                optional: false,
                def: 'nyc mocha --opts mocha.opts --watch "./{,!(node_modules)/**/}*.[sS]pec.<ext-glob>"'
              }
            },
            jest: {
              default: {
                type: 'string',
                optional: false,
                def: 'jest --config=jest.config.js'
              },
              watch: {
                type: 'string',
                optional: false,
                def: 'jest --config=jest.config.js --watch'
              },
              coverageDefault: {
                type: 'string',
                optional: false,
                def: 'jest --config=jest.config.js --coverage'
              },
              coverageWatch: {
                type: 'string',
                optional: false,
                def: 'jest --config=jest.config.js --coverage --watch'
              }
            }
          }
        }
      }
    },
    serve: {
      type: ['object', 'boolean'],
      optional: false,
      $acceptOnly: 'false',
      $mapDef: true,
      properties: {
        demoDir: {
          type: 'string',
          optional: false,
          def: 'demo/watch'
        },
        helperDir: {
          type: 'string',
          optional: false,
          def: 'demo/helper'
        },
        serveDir: {
          type: 'array',
          optional: false,
          items: {
            type: 'string'
          },
          def: []
        },
        open: {
          type: 'boolean',
          optional: false,
          def: true
        },
        port: {
          type: 'number',
          optional: false,
          gt: 0,
          lt: 65535,
          def: 4000
        }
      }
    },
    license: {
      type: 'object',
      optional: false,
      $mapDef: true,
      properties: {
        banner: {
          type: 'boolean',
          optional: false,
          def: true
        }
      }
    },
    format: {
      type: 'object',
      optional: false,
      $mapDef: true,
      properties: {
        extensions: {
          type: 'array',
          optional: false,
          def: ['js', 'jsx', 'ts', 'tsx', 'html', 'scss', 'css', 'less', 'json'],
          items: {
            type: 'string'
          }
        },
        advanced: {
          optional: false,
          $mapDef: true,
          properties: {
            command: {
              type: 'string',
              optional: false,
              def: 'prettier --write **/*.{<ext-glob>}'
            }
          }
        }
      }
    },
    lint: {
      type: 'object',
      optional: false,
      $mapDef: true,
      properties: {
        advanced: {
          optional: false,
          $mapDef: true,
          properties: {
            style: {
              type: 'string',
              optional: false,
              def: 'stylelint **/*.{styl,scss,sass,less,css}'
            },
            typescript: {
              type: 'string',
              optional: false,
              def: 'tslint **/*.{ts,tsx}'
            },
            javascript: {
              type: 'string',
              optional: false,
              def: 'eslint **/*.{js,mjs}'
            }
          }
        }
      }
    }
  }
};
/**
 * Recursively map object defaults.
 * @param schema Inspector schema.
 */

var mapObjectDefaults = schema => {
  var obj = {};
  forOwn(schema.properties, (value, key) => {
    if (value.def) {
      obj[key] = value.def;
    } else if (value.properties) {
      obj[key] = mapObjectDefaults(value);
    }
  });
  return obj;
};
/**
 * Extend schema inspector sanitizations.
 */


inspector__default.Sanitization.extend({
  /**
   * Map object defaults if invalid.
   * @param schema Inspector schema.
   * @param candidate Current sanitization candidate.
   */
  // Do not use arrow functions for this.
  // tslint:disable-next-line
  mapDef: function mapDef(schema, candidate) {
    var invalidType;

    if (Array.isArray(schema.type)) {
      invalidType = !schema.type.includes(typeof candidate);
    } else {
      invalidType = typeof candidate !== schema.type;
    }

    if (schema.$mapDef && candidate === null && invalidType) {
      return mapObjectDefaults(schema);
    }

    return candidate;
  },

  /**
   * Set accept only boolean value if not valid.
   * @param schema Inspector schema.
   * @param candidate Current sanitization candidate.
   */
  // Do not use arrow functions for this.
  // tslint:disable-next-line
  acceptOnly: function acceptOnly(schema, candidate) {
    if (schema.$acceptOnly && typeof candidate === 'boolean') {
      if (String(candidate) !== schema.$acceptOnly) {
        this.report("support only boolean value '".concat(schema.$acceptOnly, "' or object"));
        return mapObjectDefaults(schema);
      }
    }

    return candidate;
  }
});
/**
 * Extend schema inspector validations.
 */

inspector__default.Validation.extend({
  /**
   * Validate whether candidate value is accept only boolean value.
   * @param schema Inspector schema.
   * @param candidate Current validating candidate.
   */
  // Do not use arrow functions for this.
  // tslint:disable-next-line
  acceptOnly: function acceptOnly(schema, candidate) {
    if (schema.$acceptOnly && typeof candidate === 'boolean') {
      if (String(candidate) !== schema.$acceptOnly) {
        this.report("support only boolean value '".concat(schema.$acceptOnly, "' or object"));
        return mapObjectDefaults(schema);
      }
    }
  }
});
/**
 * Cross validate configuration of dependent config errors.
 * @param packerConfig Packer configuration reference.
 * @param log Logger reference.
 */

var crossValidateConfig = (packerConfig, log) => {
  var commonMsg = 'malformed packer config (.packerrc.js):\n';
  var entryExt = path__default.extname(packerConfig.entry);

  if (packerConfig.compiler.script.preprocessor === 'typescript' && !['.ts', '.tsx'].includes(entryExt)) {
    log.error(commonMsg + "Entry extension must be '.ts' when script preprocessor is 'typescript'");
    process.exit(1);
  }

  if (packerConfig.compiler.script.preprocessor === 'none' && !['.js', '.jsx', '.es6', '.es', '.mjs'].includes(entryExt)) {
    log.error(commonMsg + "Entry extension must be '.js', '.jsx', '.es6', '.es' or '.mjs' " + "when script preprocessor is 'none'");
    process.exit(1);
  }

  if (['node', 'node-cli'].includes(packerConfig.compiler.buildMode)) {
    if (!['cjs', 'esm'].includes(packerConfig.bundle.format)) {
      log.error(commonMsg + "Bundle format must be 'cjs' or 'esm' when bundle mode is 'node' or 'node-cli'");
      process.exit(1);
    }

    if (packerConfig.serve !== false) {
      log.error(commonMsg + "Serve on watch mode is not supported when bundle format is 'cjs' or 'esm'");
      process.exit(1);
    }
  }

  if (packerConfig.compiler.buildMode === 'browser' && !['umd', 'amd', 'iife', 'system'].includes(packerConfig.bundle.format)) {
    log.error(commonMsg + "Bundle format must be 'umd', 'amd', 'iife' or 'system' when bundle mode is 'browser'");
    process.exit(1);
  }

  if (packerConfig.test.framework === 'jest' && packerConfig.test.environment === 'browser') {
    log.error(commonMsg + "Test environment 'browser' is not supported when test framework is 'jest'");
    process.exit(1);
  }

  if (['jasmine', 'mocha'].includes(packerConfig.test.framework) && packerConfig.test.environment === 'enzyme') {
    log.error(commonMsg + "Test environment 'enzyme' is only supported when test framework is 'jasmine' or 'mocha'");
    process.exit(1);
  }
};

var isWindows = process.platform === 'win32';
var args = process.argv.splice(2);
/**
 * Run shell command in node environment.
 * @param command Shell command with arguments.
 * @param dir Execution dir.
 * @param log Logger reference.
 */

var runShellCommand = (command, dir, log) => {
  var commandSegments = command.split(' ');
  var baseCmd = commandSegments.shift();
  var cmd = isWindows ? "".concat(baseCmd, ".cmd") : baseCmd;
  var options = {
    cwd: dir,
    detached: false,
    stdio: 'inherit'
  };
  log.trace('shell command execution options:\n%o', options);
  log.trace('shell command: %s %s', cmd, commandSegments.join(' '));
  return new Promise(resolve => {
    var childProcess = child_process.spawn(cmd, commandSegments, options);
    childProcess.on('close', () => {
      resolve();
    });
  });
};
/**
 * Make relative directory path by fragments.
 * Create none existing fragments and ignore existing.
 * @param fragments Directory path fragments.
 */

var makeRelativeDirPath = function makeRelativeDirPath() {
  var currentPath = process.cwd();

  for (var _len = arguments.length, fragments = new Array(_len), _key = 0; _key < _len; _key++) {
    fragments[_key] = arguments[_key];
  }

  for (var fragment of fragments) {
    currentPath = path__default.join(currentPath, fragment);

    if (!fs.existsSync(currentPath)) {
      fs.mkdirSync(currentPath);
    }
  }
};
/**
 * Write file async.
 * @param filePath Path to write the file.
 * @param data Data to include in the file being written.
 */

var writeFile = (filePath, data) => {
  return new Promise((resolve, reject) => {
    fs.writeFile(filePath, data, err => {
      if (err) {
        reject(err);
        return;
      }

      resolve();
    });
  });
};
/**
 * Read file async.
 * @param filePath Path to read the file from.
 */

var readFile = filePath => {
  return new Promise((resolve, reject) => {
    fs.readFile(filePath, (err, data) => {
      if (err) {
        reject(err);
        return;
      }

      resolve(data.toString('utf8'));
    });
  });
};
/**
 * Read configuration file sync.
 * @param filePath File path.
 */

var readConfigFile = filePath => {
  if (fs.existsSync(filePath)) {
    return JSON.parse(fs.readFileSync(filePath, 'utf-8'));
  }

  var jsPath = "".concat(filePath, ".js");

  if (fs.existsSync(jsPath)) {
    return require(jsPath);
  }

  throw Error("".concat(path__default.basename(filePath), " configuration file not found in ").concat(filePath));
};
/**
 * Merge objects deep.
 * @param object Base object.
 * @param sources Source objects to merge.
 */

var mergeDeep = function mergeDeep(object) {
  for (var _len2 = arguments.length, sources = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    sources[_key2 - 1] = arguments[_key2];
  }

  return mergeWith(object, ...sources, (objValue, srcValue) => {
    if (Array.isArray(objValue)) {
      if (Array.isArray(srcValue)) {
        return srcValue;
      }

      return [];
    }

    if (typeof objValue !== typeof sources) {
      return srcValue;
    }
  });
};
/**
 * Watch source for changes and execute callback function
 * @param watchPath Watch path or paths glob collection.
 * @param callback Callback function to execute on change
 */

var watchSource =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (watchPath, callback) {
    yield callback();
    var watcher = chokidar.watch(watchPath, {
      ignored: 'node_modules/**',
      cwd: process.cwd()
    });
    yield new Promise(() => {
      watcher.on('change',
      /*#__PURE__*/
      _asyncToGenerator(function* () {
        yield callback();
      }));
    });
  });

  return function watchSource(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Import peer dependencies dynamically on runtime.
 * @param module Module name.
 * @param log Logger reference.
 */

var requireDependency = (module, log) => {
  var requirePath = require.resolve(module, {
    paths: [path__default.join(process.cwd(), 'node_modules')]
  });

  log.trace("import '%s' module from '%s'", module, requirePath);
  return require(requirePath);
};

/**
 * Library license type.
 */
class LicenseType {}
LicenseType.MIT = 'MIT License';
LicenseType.APACHE_2 = 'Apache License 2.0';
LicenseType.MPL_2 = 'Mozilla Public License 2.0';
LicenseType.BSD_2 = 'BBSD 2-Clause FreeBSD License';
LicenseType.BSD_3 = 'BSD 3-Clause Revised License';
LicenseType.ISC = 'Internet Systems Consortium (ISC) License';
LicenseType.LGPL_3 = 'GNU Lesser General Public License v3.0 only';
LicenseType.GLP_3 = 'GNU General Public License v3.0 only';
LicenseType.UNLICENSE = 'The Unlicense';

/**
 * Parse style extension by preprocessor type.
 * @param preprocessor Style preprocessor type.
 */

var parseStylePreprocessorExtension = preprocessor => {
  switch (preprocessor) {
    case 'scss':
      return 'scss';

    case 'sass':
      return 'sass';

    case 'less':
      return 'less';

    case 'stylus':
      return 'styl';

    case 'none':
      return 'css';
  }
};
/**
 * Parse license file name by license type.
 * @param license License type.
 */

var parseLicenseType = license => {
  var licenseFileName = '';

  switch (license) {
    case LicenseType.MIT:
      licenseFileName = 'MIT';
      break;

    case LicenseType.APACHE_2:
      licenseFileName = 'Apache-2.0';
      break;

    case LicenseType.MPL_2:
      licenseFileName = 'MPL-2.0';
      break;

    case LicenseType.BSD_2:
      licenseFileName = 'BSD-2-Clause-FreeBSD';
      break;

    case LicenseType.BSD_3:
      licenseFileName = 'BSD-3-Clause';
      break;

    case LicenseType.ISC:
      licenseFileName = 'ISC';
      break;

    case LicenseType.LGPL_3:
      licenseFileName = 'LGPL-3.0';
      break;

    case LicenseType.GLP_3:
      licenseFileName = 'GPL-3.0';
      break;

    case LicenseType.UNLICENSE:
      licenseFileName = 'Unlicense';
      break;
  }

  return licenseFileName;
};
/**
 * Parse script transpiler by preprocessor type.
 * @param preprocessor Script preprocessor type.
 */

var parseScriptPreprocessorExtension = preprocessor => {
  switch (preprocessor) {
    case 'typescript':
      return 'ts';

    case 'none':
    default:
      return 'js';
  }
};
/**
 * Parse script transpiler by preprocessor type and extract extension glob.
 * @param preprocessor Script preprocessor type.
 */

var parseScriptPreprocessorExtensionGlob = preprocessor => {
  var scriptExt = parseScriptPreprocessorExtension(preprocessor);
  return "{".concat(scriptExt, ",").concat(scriptExt, "x}");
};
/**
 * Parse karma script transpiler by preprocessor type and extract extension glob.
 * @param preprocessor Script preprocessor type.
 */

var parseKarmaScriptPreprocessorExtensionGlob = preprocessor => {
  var scriptExt = parseScriptPreprocessorExtension(preprocessor);
  return "+(".concat(scriptExt, "|").concat(scriptExt, "x)");
};
/**
 * Parse package build mode.
 * @param packerOptions Packer options object.
 */

var parseBuildMode = packerOptions => {
  if (packerOptions.browserCompliant) {
    return 'browser';
  } else if (packerOptions.cliProject) {
    return 'node-cli';
  }

  return 'node';
};
/**
 * Parse test environment type.
 * @param packerOptions Packer options object.
 */

var parseTestEnvironment = packerOptions => {
  var testEnvironment = 'node';

  if (packerOptions.testFramework === 'jest' && packerOptions.reactLib && packerOptions.useEnzyme) {
    testEnvironment = 'enzyme';
  } else {
    testEnvironment = packerOptions.testEnvironment || (packerOptions.browserCompliant ? 'jsdom' : 'node');
  }

  return testEnvironment;
};

/**
 * Metadata reader class.
 */

class MetaData {
  /**
   * Fetch packer configuration object.
   * Return if already exist in cache, else read -> validate -> sanitize configuration data.
   * Configuration read logic will lookup for all extend packer config paths and merged configuration
   * from base to local order.
   * @param log Logger reference.
   */
  fetchPackerConfig(log) {
    if (this.packerConfig) {
      return;
    }

    var projectConf = require(this.readConfigPath(log)); // Read all parent packer configuration.


    var readConf = sourceConf => {
      var conf = sourceConf[sourceConf.length - 1];

      if (conf && typeof conf === 'object' && conf.extend) {
        var parentConfPath;

        if (conf.extend.startsWith('~/packer-cli')) {
          parentConfPath = path__default.join(__dirname, conf.extend.replace('~/packer-cli', '..'));
        } else {
          parentConfPath = path__default.isAbsolute(conf.extend) ? conf.extend : path__default.join(process.cwd(), conf.extend);
        }

        log.trace('read parent packer config from path: %s', parentConfPath);
        return readConf([...sourceConf, require(parentConfPath)]);
      } else {
        return sourceConf;
      }
    };

    var confCollection = readConf([projectConf]).reverse();
    var packerConfig = null;

    if (confCollection.length > 1) {
      var baseConf = confCollection.shift();
      packerConfig = mergeDeep(baseConf, ...confCollection);
    } else {
      packerConfig = confCollection[0];
    }

    log.trace('merged packer config:\n%o', packerConfig);
    var validation = inspector.validate(packerSchema, packerConfig);

    if (!validation.valid) {
      log.warn('malformed packer config (.packerrc.js):\n%s\nusing default configuration', validation.format());
    }

    var sanitizedData = inspector.sanitize(packerSchema, packerConfig).data;
    crossValidateConfig(sanitizedData, log); // Replace glob extension pattern with dynamic extensions.

    var mochaConf = sanitizedData.test.advanced.mocha;
    var formatConf = sanitizedData.format.advanced;
    var lintConf = sanitizedData.lint.advanced;
    var extGlob = parseScriptPreprocessorExtensionGlob(sanitizedData.compiler.script.preprocessor);
    mochaConf.coverageWatch = this.replaceExtensionGlob(extGlob, mochaConf.coverageWatch);
    mochaConf.coverageDefault = this.replaceExtensionGlob(extGlob, mochaConf.coverageDefault);
    mochaConf.watch = this.replaceExtensionGlob(extGlob, mochaConf.watch);
    mochaConf.default = this.replaceExtensionGlob(extGlob, mochaConf.default);
    var formatExtGlob = sanitizedData.format.extensions.join(',');
    formatConf.command = this.replaceExtensionGlob(formatExtGlob, formatConf.command);
    this.packerConfig = sanitizedData;
  }
  /**
   * Read and cache packer configuration.
   * @param log Logger reference.
   */


  readPackerConfig(log) {
    if (this.packerConfig) {
      return this.packerConfig;
    }

    this.fetchPackerConfig(log);
    return this.packerConfig;
  }
  /**
   * Read and cache local package.json file configuration.
   */


  readPackageData() {
    if (this.packageConfig) {
      return this.packageConfig;
    }

    this.packageConfig = require(path__default.join(process.cwd(), 'package.json'));
    return this.packageConfig;
  }
  /**
   * Read and cache babel configuration.
   */


  readBabelConfig() {
    if (this.babelConfig) {
      return this.babelConfig;
    }

    this.babelConfig = readConfigFile(path__default.join(process.cwd(), '.babelrc'));
    return this.babelConfig;
  }
  /**
   * Read and cache banner template.
   */


  readBannerTemplate() {
    var _this = this;

    return _asyncToGenerator(function* () {
      if (_this.bannerTemplate) {
        return _this.bannerTemplate;
      }

      _this.bannerTemplate = yield readFile(path__default.join(process.cwd(), '.packer/banner.hbs'));
      return _this.bannerTemplate;
    })();
  }
  /**
   * Read and cache packer help summary.
   */


  readPackerHelpSummary() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      if (_this2.packerHelpSummary) {
        return _this2.packerHelpSummary;
      }

      _this2.packerHelpSummary = yield readFile(path__default.join(__dirname, '../resources/dynamic/packer-help.txt'));
      return _this2.packerHelpSummary;
    })();
  }
  /**
   * Read and cache packer banner.
   */


  readPackerBanner() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      if (_this3.packerBanner) {
        return _this3.packerBanner;
      }

      _this3.packerBanner = yield readFile(path__default.join(__dirname, '../resources/dynamic/banner.txt'));
      return _this3.packerBanner;
    })();
  }
  /**
   * Read packer configuration path.
   * Use dynamic packer config path if available, else use .packerrc.js config in project root.
   * @param log Logger reference.
   */


  readConfigPath(log) {
    var dynamicConfIndex = args.findIndex(value => {
      return value.startsWith('--config') || value.startsWith('-c');
    });
    var confPath;

    if (dynamicConfIndex > -1) {
      var dynamicConf = args[dynamicConfIndex + 1];

      if (!dynamicConf) {
        log.error('config file path not defined with config option.\n%s "%s"', chalk.reset('correct packer command with'), chalk.blue('packer <command> --config path/to/packerrc.js'));
        process.exit(1);
      }

      confPath = path__default.isAbsolute(dynamicConf) ? dynamicConf : path__default.join(process.cwd(), dynamicConf);
      log.info('using dynamic packer config in %s', confPath);

      if (!fs.existsSync(confPath)) {
        log.error('packer config file not found');
        process.exit(1);
      }
    } else {
      confPath = path__default.join(process.cwd(), '.packerrc.js');

      if (!fs.existsSync(confPath)) {
        log.error('Current directory does not contain packer config.\n%s "%s %s"', chalk.reset('Generate new project via'), chalk.blue('packer generate'), chalk.green('<project name>'));
        process.exit(1);
      }
    }

    return confPath;
  }
  /**
   * Replace extension glob pattern with actual extension.
   * @param extGlob Extension glob.
   * @param command Target command.
   */


  replaceExtensionGlob(extGlob, command) {
    return command.replace('<ext-glob>', extGlob);
  }

}
/**
 * Export static meta instance.
 */


var meta = new MetaData();

/**
 * Log level.
 */
var LogLevel;

(function (LogLevel) {
  LogLevel[LogLevel["TRACE"] = 0] = "TRACE";
  LogLevel[LogLevel["INFO"] = 1] = "INFO";
  LogLevel[LogLevel["WARN"] = 2] = "WARN";
  LogLevel[LogLevel["ERROR"] = 3] = "ERROR";
  LogLevel[LogLevel["SILENT"] = 4] = "SILENT";
})(LogLevel || (LogLevel = {}));

/**
 * Get current context log level
 */

var getLogLevel = () => {
  var logLevel = LogLevel.INFO;

  if (args.includes('--info')) {
    logLevel = LogLevel.INFO;
  } else if (args.includes('--error')) {
    logLevel = LogLevel.ERROR;
  } else if (args.includes('--warn')) {
    logLevel = LogLevel.WARN;
  } else if (args.includes('--trace')) {
    logLevel = LogLevel.TRACE;
  } else if (args.includes('--silent')) {
    logLevel = LogLevel.SILENT;
  }

  return logLevel;
};
/**
 * Packer logger class.
 * Log messages depending on the log level.
 */


class Logger {
  constructor(taskName) {
    this.taskName = taskName ? "".concat(chalk.green(taskName), " ") : '';
  }
  /**
   * Create logger instance with task name.
   * @param taskName
   */


  static create(taskName) {
    return new Logger(taskName);
  }
  /**
   * Get formatted current time.
   */


  static get currentTime() {
    var d = new Date();
    var h = padStart(String(d.getHours()), 2, '0');
    var m = padStart(String(d.getMinutes()), 2, '0');
    var s = padStart(String(d.getSeconds()), 2, '0');
    var ms = padStart(String(d.getMilliseconds()), 3, '0');
    var time = "[".concat(h, ":").concat(m, ":").concat(s, ":").concat(ms, "]");
    return chalk.gray(time);
  }
  /**
   * Log error message.
   * @param message log message with console format expressions.
   * @param optionalParams Optional arguments to message.
   */


  error(message) {
    if (Logger.level <= LogLevel.ERROR) {
      for (var _len = arguments.length, optionalParams = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        optionalParams[_key - 1] = arguments[_key];
      }

      console.log("".concat(Logger.currentTime, " ").concat(this.taskName).concat(chalk.red(message)), ...optionalParams);
    }
  }
  /**
   * Log warning message.
   * @param message log message with console format expressions.
   * @param optionalParams Optional arguments to message.
   */


  warn(message) {
    if (Logger.level <= LogLevel.WARN) {
      for (var _len2 = arguments.length, optionalParams = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        optionalParams[_key2 - 1] = arguments[_key2];
      }

      console.log("".concat(Logger.currentTime, " ").concat(this.taskName).concat(chalk.yellow(message)), ...optionalParams);
    }
  }
  /**
   * Log info message.
   * @param message log message with console format expressions.
   * @param optionalParams Optional arguments to message.
   */


  info(message) {
    if (Logger.level <= LogLevel.INFO) {
      for (var _len3 = arguments.length, optionalParams = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        optionalParams[_key3 - 1] = arguments[_key3];
      }

      console.log("".concat(Logger.currentTime, " ").concat(this.taskName).concat(chalk.blue(message)), ...optionalParams);
    }
  }
  /**
   * Log trace message.
   * @param message log message with console format expressions.
   * @param optionalParams Optional arguments to message.
   */


  trace(message) {
    if (Logger.level <= LogLevel.TRACE) {
      for (var _len4 = arguments.length, optionalParams = new Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
        optionalParams[_key4 - 1] = arguments[_key4];
      }

      console.log("".concat(Logger.currentTime, " ").concat(this.taskName).concat(message), ...optionalParams);
    }
  }

}
Logger.level = getLogLevel();

/**
 * Initialize clean gulp tasks
 */

function init() {
  /**
   * Temporary file clean gulp task.
   * Clean tmp directory specified in packer config.
   */
  gulp.task('tmp:clean', () => {
    var log = Logger.create('[tmp:clean]');

    try {
      log.trace('start');
      var config = meta.readPackerConfig(log);
      return gulp.src([path__default.join(process.cwd(), config.tmp)], {
        allowEmpty: true,
        read: false
      }).pipe(clean());
    } catch (e) {
      log.error('task failure:\n', e.stack || e.message);
      process.exit(1);
    }
  });
  /**
   * Watch build temporary file clean gulp task.
   * Clean tmp directory specified in packer config.
   */

  gulp.task('watch:tmp:clean', () => {
    var log = Logger.create('[watch:tmp:clean]');

    try {
      log.trace('start');
      var config = meta.readPackerConfig(log);
      return gulp.src([path__default.join(process.cwd(), config.tmp, 'watch')], {
        allowEmpty: true,
        read: false
      }).pipe(clean());
    } catch (e) {
      log.error('task failure:\n', e.stack || e.message);
      process.exit(1);
    }
  });
  /**
   * Build distribution clean gulp task.
   * Clean dist directory specified in packer config.
   */

  gulp.task('build:clean', () => {
    var log = Logger.create('[build:clean]');

    try {
      log.trace('start');
      var config = meta.readPackerConfig(log);
      return gulp.src([path__default.join(process.cwd(), config.dist)], {
        allowEmpty: true,
        read: false
      }).pipe(clean());
    } catch (e) {
      log.error('task failure:\n', e.message, e.stack);
      process.exit(1);
    }
  });
  /**
   * Clean all temporary and distribution artifacts gulp task.
   * Clean temporary and build artifact directories on parallel mode.
   */

  gulp.task('clean', gulp.parallel('tmp:clean', 'build:clean'));
}

/**
 * Gulp handlebars runtime plugin.
 * @param data Template data to complile template with.
 * @param options Plugin options.
 */

function gulpHbsRuntime(data, options) {
  return through.obj((chunk, enc, callback) => {
    try {
      if (chunk.isNull() || chunk.extname !== '.hbs') {
        return callback(null, chunk);
      }

      var template = handlebars.compile(String(chunk.contents));
      var renderedTemplate = template(data);

      if (chunk.isBuffer()) {
        chunk.contents = Buffer.from(renderedTemplate);
      }

      if (chunk.isStream()) {
        var fileStream = through();
        fileStream.write(renderedTemplate);
        chunk.contents = fileStream;
      }

      if (options) {
        if (typeof options.replaceExt !== 'undefined') {
          chunk.path = chunk.path.replace('.hbs', options.replaceExt);
        }

        if (typeof options.rename !== 'undefined') {
          chunk.path = path__default.join(path__default.dirname(chunk.path), options.rename);
        }
      }

      callback(null, chunk);
    } catch (err) {
      console.error(err);
      throw new PluginError('gulp-hbs-runtime', 'Data parse failure!');
    }
  });
}

const b = /^(b|B)$/,
	symbol = {
		iec: {
			bits: ["b", "Kib", "Mib", "Gib", "Tib", "Pib", "Eib", "Zib", "Yib"],
			bytes: ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"]
		},
		jedec: {
			bits: ["b", "Kb", "Mb", "Gb", "Tb", "Pb", "Eb", "Zb", "Yb"],
			bytes: ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"]
		}
	},
	fullform = {
		iec: ["", "kibi", "mebi", "gibi", "tebi", "pebi", "exbi", "zebi", "yobi"],
		jedec: ["", "kilo", "mega", "giga", "tera", "peta", "exa", "zetta", "yotta"]
	};

function filesize (arg, descriptor = {}) {
	if (isNaN(arg)) {
		throw new TypeError("Invalid number");
	}

	let result = [],
		val = 0,
		bits = descriptor.bits === true,
		unix = descriptor.unix === true,
		base = descriptor.base || 2,
		round = descriptor.round !== void 0 ? descriptor.round : unix ? 1 : 2,
		locale = descriptor.locale !== void 0 ? descriptor.locale : "",
		localeOptions = descriptor.localeOptions || {},
		separator = descriptor.separator !== void 0 ? descriptor.separator : "",
		spacer = descriptor.spacer !== void 0 ? descriptor.spacer : unix ? "" : " ",
		symbols = descriptor.symbols || {},
		standard = base === 2 ? descriptor.standard || "jedec" : "jedec",
		output = descriptor.output || "string",
		full = descriptor.fullform === true,
		fullforms = descriptor.fullforms instanceof Array ? descriptor.fullforms : [],
		e = descriptor.exponent !== void 0 ? descriptor.exponent : -1,
		num = Number(arg),
		neg = num < 0,
		ceil = base > 2 ? 1000 : 1024;

	// Flipping a negative number to determine the size
	if (neg) {
		num = -num;
	}

	// Determining the exponent
	if (e === -1 || isNaN(e)) {
		e = Math.floor(Math.log(num) / Math.log(ceil));

		if (e < 0) {
			e = 0;
		}
	}

	// Exceeding supported length, time to reduce & multiply
	if (e > 8) {
		e = 8;
	}

	if (output === "exponent") {
		return e;
	}

	// Zero is now a special case because bytes divide by 1
	if (num === 0) {
		result[0] = 0;
		result[1] = unix ? "" : symbol[standard][bits ? "bits" : "bytes"][e];
	} else {
		val = num / (base === 2 ? Math.pow(2, e * 10) : Math.pow(1000, e));

		if (bits) {
			val = val * 8;

			if (val >= ceil && e < 8) {
				val = val / ceil;
				e++;
			}
		}

		result[0] = Number(val.toFixed(e > 0 ? round : 0));

		if (result[0] === ceil && e < 8 && descriptor.exponent === void 0) {
			result[0] = 1;
			e++;
		}

		result[1] = base === 10 && e === 1 ? bits ? "kb" : "kB" : symbol[standard][bits ? "bits" : "bytes"][e];

		if (unix) {
			result[1] = standard === "jedec" ? result[1].charAt(0) : e > 0 ? result[1].replace(/B$/, "") : result[1];

			if (b.test(result[1])) {
				result[0] = Math.floor(result[0]);
				result[1] = "";
			}
		}
	}

	// Decorating a 'diff'
	if (neg) {
		result[0] = -result[0];
	}

	// Applying custom symbol
	result[1] = symbols[result[1]] || result[1];

	if (locale === true) {
		result[0] = result[0].toLocaleString();
	} else if (locale.length > 0) {
		result[0] = result[0].toLocaleString(locale, localeOptions);
	} else if (separator.length > 0) {
		result[0] = result[0].toString().replace(".", separator);
	}

	// Returning Array, Object, or String (default)
	if (output === "array") {
		return result;
	}

	if (full) {
		result[1] = fullforms[e] ? fullforms[e] : fullform[standard][e] + (bits ? "bit" : "byte") + (result[0] === 1 ? "" : "s");
	}

	if (output === "object") {
		return {value: result[0], symbol: result[1]};
	}

	return result.join(spacer);
}

// Partial application for functional programming
filesize.partial = opt => arg => filesize(arg, opt);

/**
 * Get distribution banner comment.
 * @param packerConfig Packer configuration object.
 * @param packageConfig Package configuration object.
 */

var getBanner =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (packerConfig, packageConfig) {
    if (packerConfig.license.banner) {
      var bannerTemplate = yield meta.readBannerTemplate();
      return handlebars.compile(bannerTemplate)({
        config: packerConfig,
        pkg: packageConfig
      });
    }
  });

  return function getBanner(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Get rollup base configuration.
 * @param packerConfig Packer configuration object.
 * @param packageConfig Package configuration object.
 * @param banner Banner comment.
 * @param sourceMap If true, a separate sourcemap file will be created. If inline, the sourcemap will be appended to
 * the resulting output file as a data URI.
 */

var getBaseConfig = (packerConfig, packageConfig, banner, sourceMap) => {
  return {
    input: path__default.join(packerConfig.source, packerConfig.entry),
    output: {
      banner,
      name: packageConfig.name,
      sourcemap: typeof sourceMap !== 'undefined' ? sourceMap : packerConfig.compiler.sourceMap
    }
  };
};
/**
 * Get rollup style build plugins.
 * @param packerConfig Packer configuration object.
 * @param packageConfig Package configuration object.
 * @param watch Copy stylesheets artifacts to watch temp directory if true, else dist directory.
 * @param main Indicate whether it is the main build task to emit stylesheets. Otherwise, ignore style sheet building
 * unless style inline mode is enabled.
 * @param log Logger reference.
 */

var getStyleBuildPlugins = (packerConfig, packageConfig, watch, main, log) => {
  if (!packerConfig.compiler.style) {
    log.trace('style build disabled');
    return [];
  }

  if (!main && !packerConfig.compiler.style.inline) {
    log.trace('ignore style imports to avoid redundant style compilation');
    return [rollupIgnoreImport(mergeDeep({
      exclude: 'node_modules/**',
      extensions: ['.scss', '.sass', '.styl', '.css', '.less']
    }, packerConfig.compiler.advanced.rollup.pluginOptions.ignoreImport))];
  }

  log.trace('init style build plugins');
  var styleDir = watch ? path__default.join(packerConfig.tmp, 'watch') : packerConfig.dist;
  var fileName = packageConfig.name + '.css';
  var styleDist = path__default.join(process.cwd(), styleDir, packerConfig.compiler.style.outDir, fileName);
  log.trace('stylesheet dist file name %s', styleDist);
  return [rollupPostCss(mergeDeep({
    exclude: 'node_modules/**',
    extract: packerConfig.compiler.style.inline ? false : styleDist,
    config: {
      path: path__default.join(process.cwd(), 'postcss.config.js'),
      ctx: {
        config: packerConfig
      }
    },
    // disable sourcemaps when inline bundling
    sourceMap: packerConfig.compiler.style.inline ? false : packerConfig.compiler.sourceMap
  }, packerConfig.compiler.advanced.rollup.pluginOptions.postCss))];
};
/**
 * Generate minified style sheets.
 * @param packerConfig Packer configuration object.
 * @param packageConfig Package configuration object.
 * @param log Logger reference.
 */

var generateMinStyleSheet =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(function* (packerConfig, packageConfig, log) {
    if (!packerConfig.compiler.style || packerConfig.compiler.style.inline) {
      return;
    }

    var nano = requireDependency('cssnano', log);
    var srcPath = path__default.join(process.cwd(), packerConfig.dist, packerConfig.compiler.style.outDir);
    var srcFileName = "".concat(packageConfig.name, ".css");
    var srcFilePath = path__default.join(srcPath, srcFileName);
    var srcMinFileName = "".concat(packageConfig.name, ".min.css");
    var srcMinFilePath = path__default.join(srcPath, srcMinFileName);
    var srcCss = yield readFile(srcFilePath);
    var options = {
      from: srcFileName,
      to: srcMinFileName
    }; // only set cssnano sourcemap options when sourcemap is true

    if (packerConfig.compiler.sourceMap === true) {
      var srcMapFilePath = path__default.join(srcPath, "".concat(packageConfig.name, ".css.map"));
      var srcMapCss = yield readFile(srcMapFilePath);
      options.map = {
        sourcesContent: true,
        prev: srcMapCss
      };
    }

    var result;
    log.trace('cssnano options:\n%o', options);

    try {
      result = yield nano.process(String(srcCss), mergeDeep(options, packerConfig.compiler.advanced.other.cssnano));
    } catch (e) {
      log.error('style minify failure:\n%s', e.stack || e.message);
      process.exit(1);
    }

    yield writeFile(srcMinFilePath, result.css); // save sourcemap file only when sourcemap true

    if (packerConfig.compiler.sourceMap === true) {
      var srcMinMapFilePath = path__default.join(srcPath, "".concat(srcMinFileName, ".map"));
      yield writeFile(srcMinMapFilePath, result.map);
    }
  });

  return function generateMinStyleSheet(_x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * Get rollup dependency resolve plugins.
 * @param packerConfig  Packer configuration object.
 */

var getDependencyResolvePlugins = packerConfig => {
  var mainFields = ['module', 'jsnext', 'main'];

  if (packerConfig.compiler.buildMode) {
    mainFields.push('browser');
  }

  var plugins = [rollupIgnore(packerConfig.ignore), rollupResolve(mergeDeep({
    mainFields,
    extensions: ['.mjs', '.js', '.jsx', '.json', '.node'],
    browser: packerConfig.compiler.buildMode === 'browser',
    preferBuiltins: true
  }, packerConfig.compiler.advanced.rollup.pluginOptions.nodeResolve)), rollupCommonjs(mergeDeep({
    include: 'node_modules/**'
  }, packerConfig.compiler.advanced.rollup.pluginOptions.commonjs)), rollupJson(packerConfig.compiler.advanced.rollup.pluginOptions.json)]; // include global and builtin plugins only when browser build mode is enabled

  if (packerConfig.compiler.buildMode === 'browser') {
    plugins.push(rollupGlobals(packerConfig.compiler.advanced.rollup.pluginOptions.globals));
  }

  return plugins;
};
/**
 * Get script build rollup plugins.
 * @param packageModule Package module type.
 * @param generateDefinition Generate type definitions if true.
 * @param check Set to false to avoid doing any diagnostic checks on the code.
 * @param packerConfig Packer configuration object.
 * @param babelConfig Babel configuration object.
 * @param typescript Typescript compiler reference.
 * @param log  Logger reference.
 */

var getScriptBuildPlugin = (packageModule, generateDefinition, check, packerConfig, babelConfig, typescript, log) => {
  var plugins = [];

  if (packerConfig.compiler.script.preprocessor === 'typescript') {
    var buildConf = {
      check,
      tsconfig: "tsconfig.json",
      typescript,
      cacheRoot: path__default.join(process.cwd(), packerConfig.tmp, 'build', packageModule, '.rts2_cache'),
      clean: true
    };

    if (generateDefinition) {
      buildConf.tsconfigOverride = {
        compilerOptions: {
          declaration: true,
          declarationDir: path__default.join(process.cwd(), packerConfig.dist, packerConfig.compiler.script.tsd)
        }
      };
      buildConf.useTsconfigDeclarationDir = true;
    }

    plugins.push(rollupTypescript(mergeDeep(buildConf, packerConfig.compiler.advanced.rollup.pluginOptions.typescript)));
  }

  var moduleConfig = get(babelConfig, "env.".concat(packageModule));

  if (!moduleConfig) {
    log.error('%s module configuration not found in .babelrc', packageModule);
    process.exit(1);
  }

  plugins.push(rollupBabel(mergeDeep({
    babelrc: false,
    exclude: 'node_modules/**',
    extensions: ['.js', '.jsx', '.es6', '.es', '.mjs', '.ts', '.tsx'],
    plugins: moduleConfig.plugins || [],
    presets: moduleConfig.presets || [],
    runtimeHelpers: true
  }, packerConfig.compiler.advanced.rollup.pluginOptions.babel)));
  return plugins;
};
/**
 * Get pre bundle rollup plugins.
 * @param packerConfig Packer configuration object.
 */

var getPreBundlePlugins = packerConfig => {
  var plugins = [];

  if (packerConfig.replacePatterns.length) {
    plugins.push(rollupReplace(mergeDeep({
      exclude: 'node_modules/**',
      patterns: packerConfig.replacePatterns
    }, packerConfig.compiler.advanced.rollup.pluginOptions.replace)));
  }

  plugins.push(rollupHandlebars(packerConfig.compiler.advanced.rollup.pluginOptions.handlebars), rollupImage(mergeDeep({
    exclude: 'node_modules/**',
    extensions: /\.(png|jpg|jpeg|gif|svg)$/,
    limit: packerConfig.compiler.script.image.inlineLimit,
    output: path__default.join(packerConfig.dist, packerConfig.compiler.script.image.outDir)
  }, packerConfig.compiler.advanced.rollup.pluginOptions.image)));
  return plugins;
};
/**
 * Get post bundle rollup plugins.
 * @param task Gulp task name.
 * @param type Package module type.
 * @param packerConfig Packer config object.
 */

var getPostBundlePlugins = (packerConfig, task, type) => {
  if (Logger.level <= LogLevel.INFO) {
    return [rollupFilesize(mergeDeep({
      showMinifiedSize: false,
      showBrotliSize: false,
      showGzippedSize: false,
      render: (options, sourceBundle, _ref3) => {
        var {
          gzipSize,
          bundleSize
        } = _ref3;
        var bundleFormatted = "bundle size: ".concat(chalk.red(bundleSize));
        return chalk.yellow("".concat(Logger.currentTime, " ").concat(chalk.green(task), " ").concat(type, " ").concat(bundleFormatted));
      }
    }, packerConfig.compiler.advanced.rollup.pluginOptions.filesize))];
  }

  return [];
};
/**
 * Get custom rollup plugins included via packer config.
 * @param packerConfig Packer configuration object.
 * @param type Package module type.
 */

var customRollupPlugins = (packerConfig, type) => {
  if (packerConfig.compiler.customRollupPluginExtractor) {
    return packerConfig.compiler.customRollupPluginExtractor(type, packerConfig);
  }

  return [];
};
/**
 * Generate bundled artifacts via rollup.
 * @param packerConfig Packer configuration object.
 * @param packageConfig Package configuration object.
 * @param bundleConfig Rollup bundle configuration object.
 * @param type Package module type.
 * @param minify Generate minified bundle artifact if true.
 * @param trackBuildPerformance Track rollup build performance.
 * @param log Logger reference.
 */

var generateBundle =
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(function* (packerConfig, packageConfig, bundleConfig, type, minify, trackBuildPerformance, log) {
    log.trace('%s bundle build start', type);
    var bundle = yield rollup.rollup(mergeDeep(bundleConfig, packerConfig.compiler.advanced.rollup.inputOptions));
    var outputOptions = mergeDeep(bundleConfig.output, packerConfig.compiler.advanced.rollup.outputOptions);
    var {
      output
    } = yield bundle.write(outputOptions);
    var chunks = output.filter(chunk => !chunk.isAsset);
    var {
      map,
      code
    } = chunks[0];
    log.trace('%s bundle build end', type);

    if (trackBuildPerformance) {
      // Performance stats should be available on any log level
      var messageBase = "".concat(Logger.currentTime, " ").concat(chalk.green(type));
      console.log(chalk.blue('%s bundle performance statistics:\n%o'), messageBase, bundle.getTimings());
    }

    if (minify) {
      log.trace('%s minified bundle build start', type);
      var minFileDist = bundleConfig.output.file.replace('.js', '.min.js');
      var minMapFileDist = minFileDist.concat('.map');
      var minFileName = path__default.basename(minFileDist);
      var minMapFileName = minFileName.concat('.map');
      var minSourceMapConfig = {
        content: map,
        filename: minFileName
      };

      if (packerConfig.compiler.sourceMap === 'inline') {
        minSourceMapConfig.url = 'inline';
      } else if (packerConfig.compiler.sourceMap) {
        minSourceMapConfig.url = minMapFileName;
      } else {
        minSourceMapConfig = undefined;
      }

      var minData = terser.minify(code, mergeDeep({
        sourceMap: minSourceMapConfig,
        output: {
          comments: /@preserve|@license|@cc_on/i
        }
      }, packerConfig.compiler.advanced.other.terser));
      log.trace('%s minified bundle terser config\n%o', type, minData);

      if (minData.error) {
        var {
          message,
          line,
          col: column
        } = minData.error;
        log.error('%s bundle min build failure \n%s', type, codeFrameColumns(code, {
          start: {
            line,
            column
          }
        }, {
          message
        }));
        process.exit(1);
      } else {
        yield writeFile(minFileDist, minData.code);

        if (minSourceMapConfig && minSourceMapConfig.url === minMapFileName) {
          log.trace('%s minified bundle write source maps', type);
          yield writeFile(minMapFileDist, minData.map);
        }

        if (Logger.level <= LogLevel.INFO) {
          var formatOptions = packerConfig.compiler.advanced.rollup.pluginOptions.filesize.format;
          var size = filesize(Buffer.byteLength(minData.code), mergeDeep({
            base: 10
          }, formatOptions));
          var minType = "".concat(type, " minified size:");
          log.info(chalk.yellow(minType), chalk.red(size));
        }
      }

      log.trace('%s minified bundle build end', type);
    }
  });

  return function generateBundle(_x6, _x7, _x8, _x9, _x10, _x11, _x12) {
    return _ref4.apply(this, arguments);
  };
}();
/**
 * Get external dependency filter function.
 * @param packerConfig Packer config object.
 */

var getExternalFilter = packerConfig => {
  var filter = packerConfig.bundle.externals.map(external => glob(external));
  return id => {
    return filter.some(include => include.test(id));
  };
};
/**
 * Extract bundle external.
 * @param packerConfig Packer config object.
 */

var extractBundleExternals = packerConfig => {
  // Map externals from globals if mapExternals is true.
  return packerConfig.bundle.mapExternals ? Object.keys(packerConfig.bundle.globals) : getExternalFilter(packerConfig);
};

/**
 * Initialize build associated gulp tasks.
 */

function init$1() {
  /**
   * Copy build essentials gulp task.
   * This task will copy dynamically generated package.json file and packer config specified copy globs.
   */
  gulp.task('build:copy:essentials', () => {
    var log = Logger.create('[build:copy:essentials]');

    try {
      log.trace('start');
      var packageJson = meta.readPackageData();
      var config = meta.readPackerConfig(log);
      var targetPackage = {}; // only copy needed properties from project's package json

      config.compiler.packageFieldsToCopy.forEach(field => {
        targetPackage[field] = packageJson[field];
      });

      if (config.compiler.buildMode === 'node-cli') {
        targetPackage.bin = packageJson.bin;
      }

      if (config.compiler.build.bundleMin) {
        targetPackage.main = path__default.join('bundle', "".concat(packageJson.name, ".").concat(config.bundle.format, ".min.js"));
      } else {
        targetPackage.main = path__default.join('bundle', "".concat(packageJson.name, ".").concat(config.bundle.format, ".js"));
      }

      if (config.compiler.script.preprocessor === 'typescript') {
        targetPackage.typings = config.compiler.script.tsd ? path__default.join(config.compiler.script.tsd, 'index.d.ts') : 'index.d.ts';
      }

      if (config.compiler.build.es5Min) {
        var esm5MinPath = path__default.join('fesm5', "".concat(packageJson.name, ".esm.min.js"));
        targetPackage.module = esm5MinPath;
        targetPackage.fesm5 = esm5MinPath;
      } else if (config.compiler.build.es5) {
        var fesm5Path = path__default.join('fesm5', "".concat(packageJson.name, ".esm.js"));
        targetPackage.module = fesm5Path;
        targetPackage.fesm5 = fesm5Path;
      }

      if (config.compiler.build.esnextMin) {
        var esmNextMinPath = path__default.join('fesmnext', "".concat(packageJson.name, ".esm.min.js"));
        targetPackage.esnext = esmNextMinPath;
        targetPackage.fesmnext = esmNextMinPath;
        targetPackage['jsnext:main'] = esmNextMinPath;
      } else if (config.compiler.build.esnext) {
        var esmNextPath = path__default.join('fesmnext', "".concat(packageJson.name, ".esm.js"));
        targetPackage.esnext = esmNextPath;
        targetPackage.fesmnext = esmNextPath;
        targetPackage['jsnext:main'] = esmNextPath;
      } // Map dependencies to target package file


      switch (config.compiler.dependencyMapMode) {
        case 'cross-map-peer-dependency':
          targetPackage.peerDependencies = packageJson.dependencies;
          break;

        case 'cross-map-dependency':
          targetPackage.dependencies = packageJson.peerDependencies;
          break;

        case 'map-dependency':
          targetPackage.dependencies = packageJson.dependencies;
          break;

        case 'map-peer-dependency':
          targetPackage.peerDependencies = packageJson.peerDependencies;
          break;

        case 'all':
          targetPackage.peerDependencies = packageJson.peerDependencies;
          targetPackage.dependencies = packageJson.dependencies;
          break;
      } // copy the needed additional files in the 'dist' folder


      if (config.copy.length > 0) {
        log.trace('copy static files and package.json');
        return gulp.src(config.copy.map(copyFile => {
          return path__default.join(process.cwd(), copyFile);
        }), {
          base: process.cwd()
        }).on('error', e => {
          log.error('copy source missing: %s\n', e.stack || e.message);
          process.exit(1);
        }).pipe(gulpAdd('package.json', JSON.stringify(targetPackage, null, 2))).pipe(gulp.dest(path__default.join(process.cwd(), config.dist))).on('finish', () => {
          log.trace('end');
        });
      } else {
        log.trace('copy package.json');
        return gulpAdd('package.json', JSON.stringify(targetPackage, null, 2)).pipe(gulp.dest(path__default.join(process.cwd(), config.dist))).on('finish', () => {
          log.trace('end');
        });
      }
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  });
  /**
   * Copy bin artifacts gulp task.
   * This task will copy bin artifact only when build mode is node-cli
   */

  gulp.task('build:copy:bin', () => {
    var log = Logger.create('[build:copy:bin]');

    try {
      log.trace('start');
      var packageJson = meta.readPackageData();
      var config = meta.readPackerConfig(log);

      if (config.compiler.buildMode !== 'node-cli') {
        log.trace('not a cli project: bin copy abort');
        log.trace('start');
        return;
      }

      return gulp.src([path__default.join(process.cwd(), '.packer/bin.hbs')]).on('error', e => {
        log.error('bin source missing: %s\n', e.stack || e.message);
        process.exit(1);
      }).pipe(gulpHbsRuntime({
        packageName: packageJson.name,
        format: config.bundle.format
      }, {
        rename: "".concat(packageJson.name, ".js")
      })).pipe(chmod({
        group: {
          execute: true,
          read: true
        },
        others: {
          execute: true,
          read: true
        },
        owner: {
          execute: true,
          read: true,
          write: true
        }
      })) // Grant read and execute permission.
      .pipe(gulp.dest(path__default.join(process.cwd(), config.dist, 'bin'))).on('finish', () => {
        log.trace('end');
      });
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  });
  /**
   * Copy build artifacts gulp task.
   * Run copy essentials and bin on parallel mode.
   */

  gulp.task('build:copy', gulp.parallel('build:copy:essentials', 'build:copy:bin'));
  /**
   * Build distribution bundles gulp task.
   * This task contains the core logic to build scripts and styles via rollup.
   */

  gulp.task('build:bundle',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[build:bundle]');

    try {
      log.trace(' start');
      var packerConfig = meta.readPackerConfig(log);
      var packageConfig = meta.readPackageData();
      var babelConfig = meta.readBabelConfig();
      var banner = yield getBanner(packerConfig, packageConfig);
      var baseConfig = getBaseConfig(packerConfig, packageConfig, banner);
      var externals = extractBundleExternals(packerConfig);
      var buildTasks = [];
      var bundleFileName = "".concat(packageConfig.name, ".").concat(packerConfig.bundle.format, ".js");
      var trackBuildPerformance = args.includes('--perf') || args.includes('-P');
      var typescript = null;

      if (packerConfig.compiler.script.preprocessor === 'typescript') {
        typescript = requireDependency('typescript', log);
      } // flat bundle.


      var flatConfig = merge({}, baseConfig, {
        external: externals,
        output: {
          amd: packerConfig.bundle.amd,
          file: path__default.join(process.cwd(), packerConfig.dist, 'bundle', bundleFileName),
          format: packerConfig.bundle.format,
          globals: packerConfig.bundle.globals,
          name: packerConfig.bundle.namespace
        },
        perf: trackBuildPerformance,
        plugins: [...getStyleBuildPlugins(packerConfig, packageConfig, false, true, log), ...getPreBundlePlugins(packerConfig), ...getDependencyResolvePlugins(packerConfig), ...getScriptBuildPlugin('bundle', true, true, packerConfig, babelConfig, typescript, log), ...getPostBundlePlugins(packerConfig, '[build:bundle]', 'bundle'), ...customRollupPlugins(packerConfig, 'bundle')]
      });
      log.trace('flat bundle rollup config:\n%o', flatConfig);
      buildTasks.push(generateBundle(packerConfig, packageConfig, flatConfig, 'bundle', packerConfig.compiler.build.bundleMin, trackBuildPerformance, log));

      if (packerConfig.compiler.build.es5) {
        // FESM+ES5 flat module bundle.
        var es5config = merge({}, baseConfig, {
          external: getExternalFilter(packerConfig),
          output: {
            file: path__default.join(process.cwd(), packerConfig.dist, 'fesm5', "".concat(packageConfig.name, ".esm.js")),
            format: 'esm'
          },
          perf: trackBuildPerformance,
          plugins: [...getStyleBuildPlugins(packerConfig, packageConfig, false, false, log), ...getPreBundlePlugins(packerConfig), ...getDependencyResolvePlugins(packerConfig), ...getScriptBuildPlugin('es5', false, true, packerConfig, babelConfig, typescript, log), ...getPostBundlePlugins(packerConfig, '[build:bundle]', 'es5'), ...customRollupPlugins(packerConfig, 'es5')]
        });
        log.trace('es5 bundle rollup config:\n%o', es5config);
        buildTasks.push(generateBundle(packerConfig, packageConfig, es5config, 'es5', packerConfig.compiler.build.es5Min, trackBuildPerformance, log));
      }

      if (packerConfig.compiler.build.esnext) {
        // FESM+ESNEXT flat module bundle.
        var esnextConfig = merge({}, baseConfig, {
          external: getExternalFilter(packerConfig),
          output: {
            file: path__default.join(process.cwd(), packerConfig.dist, 'fesmnext', "".concat(packageConfig.name, ".esm.js")),
            format: 'esm'
          },
          perf: trackBuildPerformance,
          plugins: [...getStyleBuildPlugins(packerConfig, packageConfig, false, false, log), ...getPreBundlePlugins(packerConfig), ...getDependencyResolvePlugins(packerConfig), ...getScriptBuildPlugin('esnext', false, true, packerConfig, babelConfig, typescript, log), ...getPostBundlePlugins(packerConfig, '[build:bundle]', 'esnext'), ...customRollupPlugins(packerConfig, 'esnext')]
        });
        log.trace('esnext bundle rollup config:\n%o', esnextConfig);
        buildTasks.push(generateBundle(packerConfig, packageConfig, esnextConfig, 'esnext', packerConfig.compiler.build.esnextMin, trackBuildPerformance, log));
      }

      if (packerConfig.compiler.concurrentBuild) {
        yield Promise.all(buildTasks);
      } else {
        for (var task of buildTasks) {
          yield task;
        }
      }

      yield generateMinStyleSheet(packerConfig, packageConfig, log);
      log.trace('end');
    } catch (e) {
      log.error('task failure:\n%s', e.stack || e.message);
      process.exit(1);
    }
  }));
  /**
   * Library build gulp task.
   * Clean distribution directory and run build copy and bundle tasks on parallel mode.
   */

  gulp.task('build', gulp.series('build:clean', gulp.parallel('build:copy', 'build:bundle')));
}

/**
 * Parse package dependency map mode.
 * @param packerOptions Packer options object.
 */

var parseDependencyMapMode = packerOptions => {
  if (packerOptions.cliProject) {
    return 'map-dependency';
  }

  return 'cross-map-peer-dependency';
};
/**
 * Parse package bundle format.
 * @param packerOptions Packer options object.
 */

var parseBundleFormat = packerOptions => {
  if (packerOptions.browserCompliant) {
    return packerOptions.bundleFormat || 'umd';
  }

  return 'cjs';
};
/**
 * Generate and copy packer configuration file.
 * @param packerOptions Packer options object.
 * @param scriptExt Script file extension.
 * @param buildMode Package build mode.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 */

var copyPackerConfig = (packerOptions, scriptExt, buildMode, testEnvironment, projectDir) => {
  var entryFile = "index.".concat(scriptExt);
  var mapMode = parseDependencyMapMode(packerOptions);
  var bundleFormat = parseBundleFormat(packerOptions);
  var packerTemplatePath = path__default.join(__dirname, '../resources/dynamic/.packerrc.js.hbs');
  return () => {
    return gulp.src([packerTemplatePath]).pipe(gulpHbsRuntime({
      entry: entryFile,
      buildMode,
      scriptPreprocessor: packerOptions.scriptPreprocessor,
      styleSupport: packerOptions.styleSupport,
      inlineStyle: packerOptions.bundleStyles,
      stylePreprocessor: packerOptions.stylePreprocessor,
      isMocha: packerOptions.testFramework === 'mocha',
      isReactLib: packerOptions.reactLib,
      bundleFormat,
      namespace: packerOptions.namespace,
      amdId: packerOptions.amdId,
      testFramework: packerOptions.testFramework,
      testEnvironment,
      serveSupport: packerOptions.browserCompliant,
      dependencyMapMode: mapMode
    }, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy license file.
 * @param packerOptions Packer options object.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var licenseCopy = (packerOptions, projectDir, log) => {
  log.trace('copy license file');
  var license = parseLicenseType(packerOptions.license);
  var licenseFilePath = path__default.join(__dirname, '../resources/dynamic/license', "".concat(license, ".hbs"));
  log.trace('license file path: %s', licenseFilePath);
  var templateData = {
    year: packerOptions.year,
    author: packerOptions.author
  };
  log.trace('template data: %o', templateData);
  return () => {
    return gulp.src([licenseFilePath]).on('error', e => {
      log.error('missing license file template: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime(templateData, {
      rename: 'LICENSE'
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy readme.md file.
 * @param packageConfig Package configuration object.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyReadme = (packageConfig, projectDir, log) => {
  log.trace('copy base dynamic and static config');
  var templateData = {
    packageName: packageConfig.name,
    packageDescription: packageConfig.description
  };
  log.trace('template data: %o', templateData);
  return () => {
    return gulp.src([path__default.join(__dirname, '../resources/dynamic/README.md.hbs')]).on('error', e => {
      log.error('missing README.md template: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime(templateData, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy babel configuration.
 * @param packerOptions Packer options object.
 * @param buildMode Project build mode.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyBabelConfig = (packerOptions, buildMode, testEnvironment, projectDir, log) => {
  log.trace('copy babel config');
  var babelrc = path__default.join(__dirname, '../resources/dynamic/.babelrc.hbs');
  log.trace('babel config glob: %s', babelrc);
  var templateData = {
    browserCompliant: buildMode === 'browser',
    reactLib: packerOptions.reactLib,
    isBrowserEnvironment: testEnvironment === 'browser',
    cjsTestModule: packerOptions.testFramework === 'jasmine' || packerOptions.testFramework === 'mocha'
  };
  log.trace('template data: %o', templateData);
  return () => {
    return gulp.src([babelrc]).pipe(gulpHbsRuntime(templateData, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy gitignore file.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyGitIgnore = (projectDir, log) => {
  log.trace('copy gitignore');
  var gitignore = path__default.join(__dirname, '../resources/dynamic/.gitignore.hbs');
  log.trace('.gitignore.hbs path: %s', gitignore);
  return () => {
    return gulp.src([gitignore]).on('error', e => {
      log.error('missing .gitignore template: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime({}, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy packer asset (banner.hbs and bin.hbs) files.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyPackerAssets = (projectDir, log) => {
  log.trace('copy packer assets');
  var banner = path__default.join(__dirname, '../resources/dynamic/packer/banner.hbs');
  log.trace('banner template path: %s', banner);
  var bin = path__default.join(__dirname, '../resources/dynamic/packer/bin.hbs');
  log.trace('bin template path: %s', bin);
  return () => {
    return gulp.src([banner, bin]).on('error', e => {
      log.error('missing packer asset: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, '.packer')));
  };
};
/**
 * Copy typescript configuration tsconfig.json and tslint.json files.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyTypescriptConfig = (projectDir, log) => {
  var tsconfig = path__default.join(__dirname, '../resources/static/tsconfig.json');
  log.trace('tsconfig.json path: %s', tsconfig);
  var tslint = path__default.join(__dirname, '../resources/static/tslint.json');
  log.trace('tslint.json path: %s', tslint);
  return () => {
    return gulp.src([tsconfig, tslint]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy postcss configuration file.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyPostCssConfig = (projectDir, log) => {
  var postCss = path__default.join(__dirname, '../resources/static/postcss.config.js');
  log.trace('karma.conf.js path: %s', postCss);
  return () => {
    return gulp.src([postCss]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy ESLint configuration file.
 * @param packerOptions Packer options object reference.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyEsLintConfig = (packerOptions, projectDir, log) => {
  var eslintrc = path__default.join(__dirname, '../resources/dynamic/.eslintrc.json.hbs');
  var eslintignore = path__default.join(__dirname, '../resources/static/.eslintignore');
  log.trace('eslintrc.json.hbs path: %s', eslintrc);
  log.trace('eslintignore path: %s', eslintignore);
  return () => {
    return gulp.src([eslintrc, eslintignore]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime({
      testFramework: packerOptions.testFramework
    }, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy style lint configuration file.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyStyleLintConfig = (projectDir, log) => {
  var stylelintrc = path__default.join(__dirname, '../resources/static/.stylelintrc.json');
  var stylelintignore = path__default.join(__dirname, '../resources/static/.stylelintignore');
  log.trace('.stylelintrc.json path: %s', stylelintrc);
  log.trace('.stylelintignore path: %s', stylelintignore);
  return () => {
    return gulp.src([stylelintrc, stylelintignore]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy common configuration (package.json, .editorconfig and travis.yml) files
 * @param packageConfig Package configuration object.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyCommonConfig = (packageConfig, projectDir, log) => {
  log.trace('copy base dynamic and static config');
  var editorConfig = path__default.join(__dirname, '../resources/static/.editorconfig');
  log.trace('.editorconfig path: %s', editorConfig);
  var travis = path__default.join(__dirname, '../resources/static/.travis.yml');
  log.trace('.travis.yml path: %s', travis);
  return () => {
    return gulp.src([editorConfig, travis]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpAdd('package.json', JSON.stringify(packageConfig, null, 2))).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy prettier configuration file.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyPrettierConfig = (projectDir, log) => {
  var prettierrc = path__default.join(__dirname, '../resources/static/.prettierrc');
  var prettierignore = path__default.join(__dirname, '../resources/static/.prettierignore');
  log.trace('.prettierrc path: %s', prettierrc);
  log.trace('.prettierignore path: %s', prettierignore);
  return () => {
    return gulp.src([prettierrc, prettierignore]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(projectDir));
  };
};
/**
 * Get project configuration file generation gulp tasks.
 * @param packerOptions Packer configuration object.
 * @param packageConfig Package configuration object.
 * @param buildMode Project build mode.
 * @param scriptExt Script file extension.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var getConfigFileGenerateTasks = (packerOptions, packageConfig, buildMode, scriptExt, testEnvironment, projectDir, log) => {
  var tasks = [];

  if (packerOptions.scriptPreprocessor === 'typescript') {
    tasks.push(copyTypescriptConfig(projectDir, log));
  } else {
    tasks.push(copyEsLintConfig(packerOptions, projectDir, log));
  }

  if (packerOptions.styleSupport) {
    tasks.push(copyPostCssConfig(projectDir, log));
    tasks.push(copyStyleLintConfig(projectDir, log));
  }

  tasks.push(licenseCopy(packerOptions, projectDir, log));
  tasks.push(copyReadme(packageConfig, projectDir, log));
  tasks.push(copyBabelConfig(packerOptions, buildMode, testEnvironment, projectDir, log));
  tasks.push(copyGitIgnore(projectDir, log));
  tasks.push(copyPackerAssets(projectDir, log));
  tasks.push(copyCommonConfig(packageConfig, projectDir, log));
  tasks.push(copyPackerConfig(packerOptions, scriptExt, buildMode, testEnvironment, projectDir));
  tasks.push(copyPrettierConfig(projectDir, log));
  return tasks;
};

/**
 * Parse script transpiler by preprocessor type and extract extension glob depending on enzyme support.
 * @param packerOptions Packer options object.
 * @param scriptExt Script file extension.
 */

var parseSpecScriptExtension = (packerOptions, scriptExt) => {
  if (packerOptions.useEnzyme) {
    return "{".concat(scriptExt, ",").concat(scriptExt, "x}");
  } else {
    return scriptExt;
  }
};
/**
 * Copy jasmine configuration file.
 * @param packerOptions Packer options object.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyJasmineConfig = (packerOptions, testEnvironment, projectDir, log) => {
  var jasmine = path__default.join(__dirname, '../resources/dynamic/jasmine.json.hbs');
  log.trace('jasmine.json path: %s', jasmine);
  return () => {
    return gulp.src([jasmine]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime({
      isTypescript: packerOptions.scriptPreprocessor === 'typescript',
      useEnzyme: packerOptions.useEnzyme,
      useJsDom: testEnvironment === 'jsdom'
    }, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy jasmine helper script files.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyJasmineHelpers = (projectDir, log) => {
  var helpersGlob = path__default.join(__dirname, '../resources/dynamic/example/test/jasmine/helpers/**/*');
  log.trace('jasmine helpers path glob: %s', helpersGlob);
  return () => {
    return gulp.src([helpersGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, 'helpers')));
  };
};
/**
 * Copy jasmine test spec files.
 * @param scriptGlob Script extension glob.
 * @param scriptExt Script file extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyJasmineSpec = (scriptGlob, scriptExt, projectDir, log) => {
  var specGlob = path__default.join(__dirname, '../resources/dynamic/example/test/jasmine', scriptExt, "spec/**/*.".concat(scriptGlob));
  log.trace('jasmine spec path glob: %s', specGlob);
  return () => {
    return gulp.src([specGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, 'spec')));
  };
};
/**
 * Copy karma jasmine test spec files.
 * @param scriptGlob Script extension glob.
 * @param scriptExt Script file extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyKarmaJasmineSpec = (scriptGlob, scriptExt, projectDir, log) => {
  var specGlob = path__default.join(__dirname, '../resources/dynamic/example/test/karma/jasmine', scriptExt, "spec/**/*.".concat(scriptGlob));
  log.trace('jasmine spec path glob: %s', specGlob);
  return () => {
    return gulp.src([specGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, 'spec')));
  };
};
/**
 * Copy jasmine configuration file.
 * @param packerOptions Packer options object.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyMochaConfig = (packerOptions, testEnvironment, projectDir, log) => {
  var mocha = path__default.join(__dirname, '../resources/dynamic/mocha.opts.hbs');
  log.trace('mocha.opts path: %s', mocha);
  return () => {
    return gulp.src([mocha]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime({
      isTypescript: packerOptions.scriptPreprocessor === 'typescript',
      useEnzyme: packerOptions.useEnzyme,
      useJsDom: testEnvironment === 'jsdom'
    }, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy mocha helper script files. Used as mocha require scripts.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyMochaHelpers = (projectDir, log) => {
  var helpersGlob = path__default.join(__dirname, '../resources/dynamic/example/test/mocha/helpers/**/*');
  log.trace('mocha helpers path glob: %s', helpersGlob);
  return () => {
    return gulp.src([helpersGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, 'helpers')));
  };
};
/**
 * Copy mocha test spec files.
 * @param scriptGlob Script extension glob.
 * @param scriptExt Script file extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyMochaTestSpec = (scriptGlob, scriptExt, projectDir, log) => {
  var testGlob = path__default.join(__dirname, '../resources/dynamic/example/test/mocha', scriptExt, "test/**/*.".concat(scriptGlob));
  log.trace('mocha test spec path glob: %s', testGlob);
  return () => {
    return gulp.src([testGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, 'test')));
  };
};
/**
 * Copy karma mocha test spec files.
 * @param scriptExt Script file extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyKarmaMochaTestSpec = (scriptExt, projectDir, log) => {
  var testGlob = path__default.join(__dirname, '../resources/dynamic/example/test/karma/mocha', scriptExt, 'spec/**/*');
  log.trace('mocha test spec path glob: %s', testGlob);
  return () => {
    return gulp.src([testGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, 'test')));
  };
};
/**
 * Copy jest configuration file.
 * @param packerOptions Packer options object.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyJestConfig = (packerOptions, testEnvironment, projectDir, log) => {
  var jest = path__default.join(__dirname, '../resources/dynamic/jest.config.js.hbs');
  log.trace('jest.config.js path: %s', jest);
  return () => {
    return gulp.src([jest]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime({
      isTypescript: packerOptions.scriptPreprocessor === 'typescript',
      testEnvironment,
      useEnzyme: packerOptions.useEnzyme
    }, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy jest mock scripts.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyJestMockScripts = (projectDir, log) => {
  var mockScriptGlob = path__default.join(__dirname, '../resources/dynamic/example/test/jest/__mocks__/**/*');
  log.trace('mock script glob bath: %s', mockScriptGlob);
  return () => {
    return gulp.src([mockScriptGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, '__mocks__')));
  };
};
/**
 * Copy jest test files.
 * @param scriptGlob Script extension glob.
 * @param scriptExt Script file extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyJestTests = (scriptGlob, scriptExt, projectDir, log) => {
  var testsGlob = path__default.join(__dirname, '../resources/dynamic/example/test/jest', scriptExt, "__tests__/**/*.".concat(scriptGlob));
  log.trace('jest tests path glob: %s', testsGlob);
  return () => {
    return gulp.src([testsGlob]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path__default.join(projectDir, '__tests__')));
  };
};
/**
 * Copy test typescript configuration (tsconfig.test.json) file.
 * @param packerOptions Packer options object reference.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyTestTypescriptConfig = (packerOptions, projectDir, log) => {
  var tsconfig = path__default.join(__dirname, '../resources/dynamic/tsconfig.test.json.hbs');
  log.trace('tsconfig.test.json.hbs path: %s', tsconfig);
  return () => {
    return gulp.src([tsconfig]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulpHbsRuntime({
      isJest: packerOptions.testFramework === 'jest'
    }, {
      replaceExt: ''
    })).pipe(gulp.dest(projectDir));
  };
};
/**
 * Copy karma configuration (karma.conf.js) file.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyKarmaConfig = (projectDir, log) => {
  var karma = path__default.join(__dirname, '../resources/static/karma.conf.js');
  log.trace('karma.conf.js path: %s', karma);
  return () => {
    return gulp.src([karma]).on('error', e => {
      log.error('missing config file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(projectDir));
  };
};
/**
 * Get test specification generator gulp task functions.
 * @param packerOptions Packer options object.
 * @param scriptExt Script file extension.
 * @param testEnvironment Test environment type.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var getTestSpecGeneratorTasks = (packerOptions, scriptExt, testEnvironment, projectDir, log) => {
  var tasks = [];
  var scriptGlob = parseSpecScriptExtension(packerOptions, scriptExt);

  if (packerOptions.testFramework === 'jest') {
    tasks.push(copyJestConfig(packerOptions, testEnvironment, projectDir, log));
    tasks.push(copyJestMockScripts(projectDir, log));
    tasks.push(copyJestTests(scriptGlob, scriptExt, projectDir, log));

    if (packerOptions.scriptPreprocessor === 'typescript') {
      tasks.push(copyTestTypescriptConfig(packerOptions, projectDir, log));
    }
  } else {
    if (testEnvironment === 'browser') {
      tasks.push(copyKarmaConfig(projectDir, log));

      if (packerOptions.testFramework === 'jasmine') {
        tasks.push(copyKarmaJasmineSpec(scriptGlob, scriptExt, projectDir, log));
      }

      if (packerOptions.testFramework === 'mocha') {
        tasks.push(copyKarmaMochaTestSpec(scriptExt, projectDir, log));
      }
    } else {
      if (packerOptions.scriptPreprocessor === 'typescript') {
        tasks.push(copyTestTypescriptConfig(packerOptions, projectDir, log));
      }

      if (packerOptions.testFramework === 'jasmine') {
        tasks.push(copyJasmineConfig(packerOptions, testEnvironment, projectDir, log));
        tasks.push(copyJasmineHelpers(projectDir, log));
        tasks.push(copyJasmineSpec(scriptGlob, scriptExt, projectDir, log));
      }

      if (packerOptions.testFramework === 'mocha') {
        tasks.push(copyMochaConfig(packerOptions, testEnvironment, projectDir, log));
        tasks.push(copyMochaHelpers(projectDir, log));
        tasks.push(copyMochaTestSpec(scriptGlob, scriptExt, projectDir, log));
      }
    }
  }

  return tasks;
};

/**
 * Copy example style sheets.
 * @param styleExt Style sheet extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyExampleStyleSheets = (styleExt, projectDir, log) => {
  log.trace('copy styles');
  var styleGlob = path.join(__dirname, '../resources/dynamic/example/common/style', "**/*.".concat(styleExt));
  log.trace('style glob: %s', styleGlob);
  return () => {
    return gulp.src([styleGlob]).pipe(gulp.dest(path.join(projectDir, 'src/style')));
  };
};
/**
 * Copy example handlebars templates.
 * @param project DirProject root directory.
 * @param log Logger reference.
 */

var copyExampleTemplates = (projectDir, log) => {
  log.trace('copy templates');
  var templatePath = path.join(__dirname, '../resources/dynamic/example/common/templates/**/*');
  log.trace('template path: %s', templatePath);
  return () => {
    return gulp.src([templatePath]).pipe(gulp.dest(path.join(projectDir, 'src/templates')));
  };
};
/**
 * Copy example assets.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyExampleAsset = (projectDir, log) => {
  log.trace('copy assets');
  var assetsPath = path.join(__dirname, '../resources/dynamic/example/common/assets/**/*');
  log.trace('assets path: %s', assetsPath);
  return () => {
    return gulp.src([assetsPath]).pipe(gulp.dest(path.join(projectDir, 'src/assets')));
  };
};
/**
 * Copy example source.
 * @param packerOptions Packer options object.
 * @param buildMode Package build mode.
 * @param scriptExt Script file extension.
 * @param styleExt Style sheet extension.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyExampleSource = (packerOptions, buildMode, scriptExt, styleExt, projectDir, log) => {
  log.trace('copy source');
  var fileExtensions = [scriptExt, 'hbs'];

  if (packerOptions.reactLib) {
    fileExtensions.push("".concat(scriptExt, "x"));
  }

  var exampleGlob = path.join(__dirname, '../resources/dynamic/example', scriptExt, "**/*.{".concat(fileExtensions.join(','), "}"));
  log.trace('example glob: %s', exampleGlob);
  var templateData = {
    styleExt,
    styleSupport: packerOptions.styleSupport,
    cliProject: buildMode === 'node-cli',
    reactLib: packerOptions.reactLib,
    jasmineEnzyme: packerOptions.useEnzyme && packerOptions.testFramework === 'jasmine'
  };
  log.trace('template data: %o', templateData);
  return () => {
    return gulp.src([exampleGlob]).pipe(gulpHbsRuntime(templateData, {
      replaceExt: ''
    })).pipe(gulp.dest(path.join(projectDir, 'src')));
  };
};
/**
 * Get example source generation gulp tasks.
 * @param packerOptions Packer options object.
 * @param styleExt Style sheet file extension.
 * @param scriptExt Script file extension.
 * @param buildMode Project build mode.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var getExampleSourceGenerationsTasks = (packerOptions, styleExt, scriptExt, buildMode, projectDir, log) => {
  var tasks = [copyExampleAsset(projectDir, log), copyExampleSource(packerOptions, buildMode, scriptExt, styleExt, projectDir, log)];

  if (!packerOptions.reactLib) {
    tasks.push(copyExampleTemplates(projectDir, log));
  }

  if (packerOptions.styleSupport) {
    tasks.push(copyExampleStyleSheets(styleExt, projectDir, log));
  }

  return tasks;
};

/**
 * Copy demo helper require.js script.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyDemoHelperRequireJs = (projectDir, log) => {
  log.trace('copy demo helper require.min.js');
  var srcPath = path.join(__dirname, '../resources/dynamic/demo/helper/require.min.js');
  log.trace('require.min.js path: %s', srcPath);
  return () => {
    return gulp.src([srcPath]).on('error', e => {
      log.error('missing helper file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path.join(projectDir, 'demo/helper')));
  };
};
/**
 * Copy demo helper system.js script.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyDemoHelperSystemJs = (projectDir, log) => {
  log.trace('copy demo helper system.min.js');
  var srcPath = path.join(__dirname, '../resources/dynamic/demo/helper/system.min.js');
  log.trace('require.min.js path: %s', srcPath);
  return () => {
    return gulp.src([srcPath]).on('error', e => {
      log.error('missing helper file: %s\n', e.stack || e.message);
      process.exit(1);
    }).pipe(gulp.dest(path.join(projectDir, 'demo/helper')));
  };
};
/**
 * Copy demo source.
 * HTML entry files if browser compliant, else javascript files.
 * @param packerOptions Packer options object.
 * @param buildMode Project build mode.
 * @param packageName Target package name.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var copyDemoSource = (packerOptions, buildMode, packageName, projectDir, log) => {
  log.trace('copy demo resources');
  var isAmd = packerOptions.bundleFormat === 'amd';
  var isIife = packerOptions.bundleFormat === 'umd' || packerOptions.bundleFormat === 'iife';
  var isSystem = packerOptions.bundleFormat === 'system';
  var templateGlob = buildMode === 'browser' ? '*.html.hbs' : '*.js.hbs';
  var demoTemplateGlob = path.join(__dirname, '../resources/dynamic/demo/**', templateGlob);
  log.trace('demo template glob: %s', demoTemplateGlob);
  var templateData = {
    projectName: packageName,
    includeStyles: packerOptions.styleSupport && packerOptions.bundleStyles,
    namespace: packerOptions.namespace,
    watchDir: '.tmp/watch',
    distDir: 'dist',
    require: isAmd,
    iife: isIife,
    system: isSystem,
    format: packerOptions.bundleFormat,
    amdModule: packerOptions.amdId,
    reactLib: packerOptions.reactLib
  };
  log.trace('template data: %o', templateData);
  return () => {
    return gulp.src([demoTemplateGlob]).pipe(gulpHbsRuntime(templateData, {
      replaceExt: ''
    })).pipe(gulp.dest(path.join(projectDir, 'demo')));
  };
};
/**
 * Get demo source generation gulp tasks.
 * @param packerOptions Packer options object.
 * @param buildMode Project build mode.
 * @param packageName Package configuration object.
 * @param projectDir Project root directory.
 * @param log Logger reference.
 */

var getDemoSourceGenerationTasks = (packerOptions, buildMode, packageName, projectDir, log) => {
  var tasks = [];

  if (buildMode !== 'node-cli') {
    tasks.push(copyDemoSource(packerOptions, buildMode, packageName, projectDir, log));
  }

  if (buildMode === 'browser') {
    if (packerOptions.bundleFormat === 'system') {
      tasks.push(copyDemoHelperSystemJs(projectDir, log));
    }

    if (packerOptions.bundleFormat === 'amd') {
      tasks.push(copyDemoHelperRequireJs(projectDir, log));
    }
  }

  return tasks;
};

var name = "packer-cli";
var version = "2.17.3";
var dependencies = {
	"@babel/code-frame": "^7.5.5",
	"@babel/core": "^7.6.2",
	chalk: "^2.4.1",
	filesize: "^5.0.1",
	glob: "^7.1.4",
	"glob-to-regexp": "^0.4.1",
	gulp: "^4.0.2",
	"gulp-add": "^0.1.0",
	"gulp-chmod": "^3.0.0",
	"gulp-clean": "^0.4.0",
	handlebars: "^4.4.0",
	inquirer: "^7.0.0",
	"karma-rollup-preprocessor": "^7.0.2",
	lodash: "^4.17.15",
	"plugin-error": "^1.0.1",
	prettier: "^1.18.2",
	rollup: "^1.22.0",
	"rollup-plugin-babel": "^4.3.3",
	"rollup-plugin-browsersync": "^1.0.0",
	"rollup-plugin-commonjs": "^10.1.0",
	"rollup-plugin-filesize": "^6.2.0",
	"rollup-plugin-hbs": "^1.2.5",
	"rollup-plugin-ignore": "^1.0.5",
	"rollup-plugin-ignore-import": "^1.3.0",
	"rollup-plugin-img": "^1.1.0",
	"rollup-plugin-istanbul": "^2.0.1",
	"rollup-plugin-json": "^4.0.0",
	"rollup-plugin-node-globals": "^1.4.0",
	"rollup-plugin-node-resolve": "^5.2.0",
	"rollup-plugin-postcss": "^2.0.3",
	"rollup-plugin-re": "^1.0.7",
	"rollup-plugin-typescript2": "^0.24.3",
	"rollup-pluginutils": "^2.8.2",
	"schema-inspector": "^1.6.8",
	terser: "^4.3.4",
	through2: "^3.0.1",
	tslib: "^1.10.0",
	typescript: "^3.6.3",
	"validate-npm-package-name": "^3.0.0",
	validator: "^11.1.0"
};
var devDependencies = {
	"@babel/plugin-transform-runtime": "^7.6.2",
	"@babel/polyfill": "^7.6.0",
	"@babel/preset-env": "^7.6.2",
	"@babel/preset-react": "^7.0.0",
	"@babel/register": "^7.6.2",
	"@babel/runtime": "^7.6.2",
	"@babel/runtime-corejs3": "^7.6.2",
	"@types/browser-sync": "^2.26.1",
	"@types/chai": "^4.2.3",
	"@types/chai-enzyme": "^0.6.7",
	"@types/enzyme": "^3.10.3",
	"@types/filesize": "^4.2.0",
	"@types/gulp": "^4.0.6",
	"@types/inquirer": "^6.5.0",
	"@types/jasmine": "^3.4.1",
	"@types/jasmine-enzyme": "^7.0.0",
	"@types/jest": "^24.0.18",
	"@types/lodash": "^4.14.141",
	"@types/mocha": "^5.2.7",
	"@types/plugin-error": "^0.1.1",
	"@types/react": "^16.9.4",
	"@types/react-dom": "^16.9.1",
	"@types/through2": "^2.0.34",
	"@types/validator": "^10.11.3",
	autoprefixer: "^9.6.1",
	"babel-jest": "^24.9.0",
	chai: "^4.2.0",
	"chai-enzyme": "^1.0.0-beta.1",
	cheerio: "^1.0.0-rc.3",
	chokidar: "^3.2.1",
	"core-js": "^3.2.1",
	cssnano: "^4.1.10",
	enzyme: "^3.10.0",
	"enzyme-adapter-react-16": "^1.14.0",
	eslint: "^6.5.1",
	"eslint-config-standard": "^14.1.0",
	"eslint-plugin-import": "^2.18.2",
	"eslint-plugin-node": "^10.0.0",
	"eslint-plugin-promise": "^4.2.1",
	"eslint-plugin-standard": "^4.0.1",
	"ignore-styles": "^5.0.1",
	jasmine: "^3.5.0",
	"jasmine-core": "^3.5.0",
	"jasmine-enzyme": "^7.1.1",
	jest: "^24.9.0",
	"jest-environment-enzyme": "^7.1.1",
	"jest-enzyme": "^7.1.1",
	"jest-haste-map": "^24.9.0",
	"jest-resolve": "^24.9.0",
	jsdom: "^15.1.1",
	karma: "^4.3.0",
	"karma-chrome-launcher": "^3.1.0",
	"karma-coverage": "^2.0.1",
	"karma-jasmine": "^2.0.1",
	"karma-jasmine-html-reporter": "^1.4.2",
	"karma-mocha": "^1.3.0",
	less: "^3.10.3",
	mocha: "^6.2.1",
	"node-sass": "^4.12.0",
	nyc: "^14.1.1",
	"packer-cli": "^2.17.2",
	"postcss-url": "^8.0.0",
	puppeteer: "^1.20.0",
	react: "^16.10.1",
	"react-dom": "^16.10.1",
	stylelint: "^11.0.0",
	"stylelint-config-standard": "^19.0.0",
	stylus: "^0.54.7",
	"ts-jest": "^24.1.0",
	"ts-node": "^8.4.1",
	tslint: "^5.20.0"
};

var IstanbulConfig = {
  extension: ['.js', '.jsx', '.ts', '.tsx'],
  include: ['src/**'],
  reporter: ['lcov', 'text-summary', 'html'],
  'temp-dir': '.tmp/nyc_output'
};

var mapDependencies = dependencies$1 => {
  return dependencies$1.reduce((previous, current) => {
    return Object.assign(previous, {
      [current]: devDependencies[current] || dependencies[current]
    });
  }, {});
};
/**
 * Build package.json configuration for generated project.
 * @param packerOptions Packer options object.
 * @param testEnvironment Test environment type.
 * @param packageName Package name.
 */


var buildPackageConfig = (packerOptions, testEnvironment, packageName) => {
  var projectAuthor = '';

  if (packerOptions.author) {
    projectAuthor = packerOptions.author;

    if (packerOptions.email) {
      projectAuthor += " <".concat(packerOptions.email, ">");
    }
  }

  var projectUrl = '';
  var projectRepository = '';

  if (packerOptions.githubUsername) {
    projectUrl = "https://github.com/".concat(packerOptions.githubUsername, "/").concat(packageName);
    projectRepository = "".concat(projectUrl, ".git");
  }

  var packageConfig = {
    name: packageName,
    version: '1.0.0',
    description: packerOptions.description || '',
    keywords: packerOptions.keywords ? packerOptions.keywords.split(',') : [],
    scripts: {
      build: 'packer build',
      watch: 'packer watch',
      start: 'npm run watch',
      test: 'packer test --watch',
      'test:coverage': 'packer test --coverage --watch',
      'test:ci': 'packer test',
      'test:coverage:ci': 'packer test --coverage',
      clean: 'packer clean',
      preversion: 'npm run build',
      postversion: 'git push && git push --tags',
      prerelease: 'npm run build',
      release: 'npm publish dist',
      lint: 'packer lint',
      'lint:script': 'packer lint --script',
      format: 'packer format'
    },
    author: projectAuthor,
    repository: projectRepository,
    license: parseLicenseType(packerOptions.license),
    homepage: packerOptions.website || projectUrl,
    dependencies: {},
    devDependencies: {},
    private: false
  };
  var dependencies = ['@babel/runtime-corejs3', 'core-js'];
  var devDependencies = ['@babel/core', '@babel/preset-env', '@babel/register', '@babel/plugin-transform-runtime'];

  if (packerOptions.reactLib) {
    devDependencies.push('@babel/preset-react');
    dependencies.push('react', 'react-dom');
  } else {
    dependencies.push('handlebars');
  }

  if (packerOptions.scriptPreprocessor === 'typescript') {
    dependencies.push('tslib');
    devDependencies.push('typescript', 'tslint');

    if (packerOptions.reactLib) {
      devDependencies.push('@types/react', '@types/react-dom');
    }
  } else {
    devDependencies.push('eslint', 'eslint-config-standard', 'eslint-plugin-import', 'eslint-plugin-node', 'eslint-plugin-promise', 'eslint-plugin-standard');
  }

  if (packerOptions.testFramework === 'jest') {
    devDependencies.push('jest');
    devDependencies.push('jest-haste-map');
    devDependencies.push('jest-resolve');

    if (packerOptions.scriptPreprocessor === 'typescript') {
      devDependencies.push('@types/jest', 'ts-jest');
    } else if (packerOptions.scriptPreprocessor === 'none') {
      devDependencies.push('babel-jest', 'babel-core');
    }

    if (packerOptions.useEnzyme) {
      devDependencies.push('enzyme', 'jest-enzyme', 'jest-environment-enzyme', 'enzyme-adapter-react-16');

      if (packerOptions.scriptPreprocessor === 'typescript') {
        devDependencies.push('@types/enzyme');
      }
    }
  }

  if (packerOptions.testFramework === 'jasmine') {
    if (packerOptions.scriptPreprocessor === 'typescript') {
      devDependencies.push('@types/jasmine');

      if (testEnvironment !== 'browser') {
        devDependencies.push('ts-node');
      }
    }

    if (testEnvironment === 'browser') {
      devDependencies.push('puppeteer', 'karma', 'karma-chrome-launcher', 'karma-coverage', 'jasmine-core', 'karma-jasmine', 'karma-jasmine-html-reporter');
    } else {
      packageConfig.nyc = IstanbulConfig;
      devDependencies.push('jasmine', 'nyc', 'ignore-styles');

      if (testEnvironment === 'jsdom') {
        devDependencies.push('jsdom');
      }
    }

    if (packerOptions.useEnzyme) {
      devDependencies.push('enzyme', 'jasmine-enzyme', 'enzyme-adapter-react-16');

      if (packerOptions.scriptPreprocessor === 'typescript') {
        devDependencies.push('@types/enzyme', '@types/jasmine-enzyme');
      }
    }
  }

  if (packerOptions.testFramework === 'mocha') {
    if (packerOptions.scriptPreprocessor === 'typescript') {
      devDependencies.push('@types/chai', '@types/mocha');

      if (testEnvironment !== 'browser') {
        devDependencies.push('ts-node');
      }
    }

    if (testEnvironment === 'browser') {
      devDependencies.push('puppeteer', 'karma', 'karma-chrome-launcher', 'karma-coverage', 'karma-mocha', 'mocha', 'chai');
    } else {
      packageConfig.nyc = IstanbulConfig;
      devDependencies.push('mocha', 'chai', 'nyc', 'ignore-styles');

      if (testEnvironment === 'jsdom') {
        devDependencies.push('jsdom');
      }
    }

    if (packerOptions.useEnzyme) {
      devDependencies.push('enzyme', 'chai-enzyme', 'cheerio', 'enzyme-adapter-react-16');

      if (packerOptions.scriptPreprocessor === 'typescript') {
        devDependencies.push('@types/enzyme', '@types/chai-enzyme');
      }
    }
  }

  if (packerOptions.styleSupport) {
    packageConfig.scripts['lint:style'] = 'packer lint --style';
    devDependencies.push('stylelint', 'stylelint-config-standard', 'autoprefixer', 'postcss-url', 'cssnano');

    if (packerOptions.stylePreprocessor === 'scss' || packerOptions.stylePreprocessor === 'sass') {
      devDependencies.push('node-sass');
    }

    if (packerOptions.stylePreprocessor === 'less') {
      devDependencies.push('less');
    }

    if (packerOptions.stylePreprocessor === 'stylus') {
      devDependencies.push('stylus');
    }
  }

  packageConfig.dependencies = mapDependencies(dependencies);
  packageConfig.devDependencies = mapDependencies(devDependencies);
  packageConfig.devDependencies[name] = "^".concat(version);

  if (packerOptions.cliProject) {
    packageConfig.bin = {
      [packageName]: path.join('bin', "".concat(packageConfig.name, ".js"))
    };
  }

  return packageConfig;
};

/**
 * Initialize project generation associated gulp tasks
 */

function init$2() {
  /**
   * Project generate gulp task.
   */
  gulp.task('generate',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[generate]');

    try {
      log.trace('start');
      var questions = [{
        message: 'Give us a small description about the library (optional)?',
        name: 'description',
        type: 'input'
      }, {
        message: 'Give us a set of comma separated package keywords (optional)?',
        name: 'keywords',
        type: 'input'
      }, {
        message: "Author's name (optional)?",
        name: 'author',
        type: 'input'
      }, {
        message: "Author's email address (optional)?",
        name: 'email',
        type: 'input',
        when: answers => {
          return !!answers.author;
        },
        validate: value => {
          return !value || isEmail(value) ? true : 'Value must be a valid email address';
        }
      }, {
        message: "Author's github username (optional)?",
        name: 'githubUsername',
        type: 'input',
        validate: () => {
          return true; // todo: add GH username validation here
        }
      }, {
        message: 'Library homepage link (optional)?',
        name: 'website',
        type: 'input',
        validate: value => {
          return !value || isUrl(value) ? true : 'Value must be a valid URL';
        }
      }, {
        choices: ['none', 'typescript'],
        default: 0,
        message: "What's the script pre processor you want to use?",
        name: 'scriptPreprocessor',
        type: 'list'
      }, {
        default: true,
        message: 'Do you want style sheet support?',
        name: 'styleSupport',
        type: 'confirm'
      }, {
        choices: ['scss', 'sass', 'less', 'stylus', 'none'],
        default: 0,
        message: "What's the style pre processor you want to use?",
        name: 'stylePreprocessor',
        type: 'list',
        when: answers => {
          return answers.styleSupport;
        }
      }, {
        default: false,
        message: 'Do you want to inline bundle styles within script?',
        name: 'bundleStyles',
        type: 'confirm',
        when: answers => {
          return answers.styleSupport;
        }
      }, {
        default: true,
        message: 'Are you building a browser compliant library?',
        name: 'browserCompliant',
        type: 'confirm'
      }, {
        default: false,
        message: 'Are you building a node CLI project?',
        name: 'cliProject',
        type: 'confirm',
        when: answers => {
          return !answers.browserCompliant;
        }
      }, {
        default: false,
        message: 'Are you building a react library?',
        name: 'reactLib',
        type: 'confirm',
        when: answers => {
          return answers.browserCompliant;
        }
      }, {
        choices: ['umd', 'amd', 'iife', 'system'],
        default: 0,
        message: "What's the build bundle format you want to use?",
        name: 'bundleFormat',
        type: 'list',
        when: answers => {
          return answers.browserCompliant;
        }
      }, {
        default: 'my-lib',
        message: "What's the AMD id you want to use?",
        name: 'amdId',
        type: 'input',
        validate: value => {
          var matches = value.match(/^(?:[a-z]\d*(?:-[a-z])?)*$/i);
          return !!matches || "AMD id should only contain alphabetic characters, i.e: 'my-bundle'";
        },
        when: answers => {
          return answers.bundleFormat === 'umd' || answers.bundleFormat === 'amd';
        }
      }, {
        default: 'com.lib',
        message: "What's the library namespace you want to use?",
        name: 'namespace',
        type: 'input',
        validate: value => {
          var matches = value.match(/^(?:[a-z]\d*(?:\.[a-z])?)+$/i);
          return !!matches || "Namespace should be an object path, i.e: 'ys.nml.lib'";
        },
        when: answers => {
          return answers.bundleFormat === 'umd' || answers.bundleFormat === 'iife' || answers.bundleFormat === 'system';
        }
      }, {
        choices: ['jest', 'mocha', 'jasmine'],
        default: 0,
        message: 'Which unit test framework do you want to use?',
        name: 'testFramework',
        type: 'list',
        when: answers => {
          return answers.reactLib;
        }
      }, {
        choices: ['mocha', 'jasmine', 'jest'],
        default: 0,
        message: 'Which unit test framework do you want to use?',
        name: 'testFramework',
        type: 'list',
        when: answers => {
          return !answers.reactLib;
        }
      }, {
        default: true,
        message: 'Do you want to use enzyme to test react components?',
        name: 'useEnzyme',
        type: 'confirm',
        when: answers => {
          return answers.reactLib;
        }
      }, {
        choices: ['jsdom', 'browser'],
        default: 0,
        message: 'Choose the test environment that will be used for testing?',
        name: 'testEnvironment',
        type: 'list',
        when: answers => {
          return answers.browserCompliant && !answers.reactLib && (answers.testFramework === 'jasmine' || answers.testFramework === 'mocha');
        }
      }, {
        default: new Date().getFullYear(),
        message: "What's the library copyright year (optional)?",
        name: 'year',
        type: 'input'
      }, {
        choices: [LicenseType.MIT, LicenseType.APACHE_2, LicenseType.MPL_2, LicenseType.BSD_2, LicenseType.BSD_3, LicenseType.ISC, LicenseType.LGPL_3, LicenseType.GLP_3, LicenseType.UNLICENSE],
        default: 0,
        message: "What's the license you want to use?",
        name: 'license',
        type: 'list'
      }, {
        default: false,
        message: 'Do you want to use yarn as package manager?',
        name: 'isYarn',
        type: 'confirm'
      }];

      if (args.length < 2) {
        log.error('Please provide a library name to generate the project\n%s', chalk.blue('npx packer-cli generate my-library'));
        process.exit(1);
        return;
      }

      var packageName = args[1];
      var packageNameValidity = npmValidate(packageName);

      if (!packageNameValidity.validForNewPackages) {
        if (packageNameValidity.errors) {
          log.error('Package name error: %s', packageNameValidity.errors.join('\n'));
        } else if (packageNameValidity.warnings) {
          log.error('Package name error: %s', packageNameValidity.warnings.join('\n'));
        }

        process.exit(1);
        return;
      }

      var options = yield inquirer.prompt(questions);
      var testEnvironment = parseTestEnvironment(options);
      var packageConfig = buildPackageConfig(options, testEnvironment, packageName);
      var scriptExt = parseScriptPreprocessorExtension(options.scriptPreprocessor);
      var projectDir = path__default.join(process.cwd(), packageName);
      var styleExt = parseStylePreprocessorExtension(options.stylePreprocessor);
      var buildMode = parseBuildMode(options);
      var tasks = [...getExampleSourceGenerationsTasks(options, styleExt, scriptExt, buildMode, projectDir, log), ...getTestSpecGeneratorTasks(options, scriptExt, testEnvironment, projectDir, log), ...getDemoSourceGenerationTasks(options, buildMode, packageName, projectDir, log), ...getConfigFileGenerateTasks(options, packageConfig, buildMode, scriptExt, testEnvironment, projectDir, log)];
      yield gulp.series([gulp.parallel(tasks),
      /*#__PURE__*/
      _asyncToGenerator(function* () {
        if (!args.includes('--skipInstall') && !args.includes('-sk')) {
          yield runShellCommand(options.isYarn ? 'yarn install' : 'npm install', projectDir, log);
        }

        log.info('📦 package generated 🚀');
      })])(() => {// No implementation
      });
    } catch (e) {
      log.error('task failure\n%s', e.stack || e.message);
      process.exit(1);
    }
  }));
}

/**
 * Initialize lint gulp tasks.
 */

function init$3() {
  /**
   * Lint style source gulp task.
   * Execute style lint on source directory.
   */
  gulp.task('lint:style',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[lint:style]');

    try {
      var config = meta.readPackerConfig(log);

      if (config.compiler.style) {
        log.info('start');
        yield runShellCommand(config.lint.advanced.style, process.cwd(), log);
        log.info('end');
      } else {
        log.trace('skip style lint');
      }
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  }));
  /**
   * Lint typescript source gulp task.
   * Execute tslint on source directory.
   */

  gulp.task('lint:script:ts',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[lint:script:ts]');

    try {
      var config = meta.readPackerConfig(log);

      if (config.compiler.script.preprocessor === 'typescript') {
        log.info('start');
        yield runShellCommand(config.lint.advanced.typescript, process.cwd(), log);
        log.info('end');
      } else {
        log.trace('skip script tslint');
      }
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  }));
  /**
   * Lint javascript source gulp task.
   * Execute eslint on source directory.
   */

  gulp.task('lint:script:es',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[lint:script:es]');

    try {
      var config = meta.readPackerConfig(log);

      if (config.compiler.script.preprocessor === 'none') {
        log.info('start');
        yield runShellCommand(config.lint.advanced.javascript, process.cwd(), log);
        log.info('end');
      } else {
        log.trace('skip script eslint');
      }
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  }));
  /**
   * Lint all script files gulp task.
   * Run typescript lint and javascript lint gulp tasks on series mode.
   */

  gulp.task('lint:script', gulp.series('lint:script:ts', 'lint:script:es'));
  /**
   * Lint both styles and script files.
   * Run style lint and script lint tasks on series mode.
   */

  gulp.task('lint', gulp.series('lint:style', 'lint:script'));
}

/**
 * Initialize lint gulp tasks.
 */

function init$4() {
  /**
   * Auto format source gulp task.
   * Execute prettier on source directory.
   */
  gulp.task('format',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[format]');

    try {
      var config = meta.readPackerConfig(log);
      log.info('start');
      yield runShellCommand(config.format.advanced.command, process.cwd(), log);
      log.info('end');
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  }));
}

/**
 * Run unit test framework in node environment.
 * Support running test spec with coverage mode and normal mode.
 * @param packerConfig Packer config object.
 * @param log Logger reference.
 */

var runNodeUnitTest =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* (packerConfig, log) {
    try {
      process.env.BABEL_ENV = process.env.BABEL_ENV || 'test';
      var globScriptExtensions = parseScriptPreprocessorExtensionGlob(packerConfig.compiler.script.preprocessor);
      var watchGlob = "**/*.".concat(globScriptExtensions);
      var watchMode = args.includes('--watch') || args.includes('-W');

      if (args.includes('--coverage') || args.includes('-C')) {
        switch (packerConfig.test.framework) {
          case 'jasmine':
            if (watchMode) {
              yield watchSource(watchGlob,
              /*#__PURE__*/
              _asyncToGenerator(function* () {
                yield runShellCommand(packerConfig.test.advanced.jasmine.coverageWatch, process.cwd(), log);
              }));
            } else {
              yield runShellCommand(packerConfig.test.advanced.jasmine.coverageDefault, process.cwd(), log);
            }

            break;

          case 'mocha':
            if (watchMode) {
              yield runShellCommand(packerConfig.test.advanced.mocha.coverageWatch, process.cwd(), log);
            } else {
              yield runShellCommand(packerConfig.test.advanced.mocha.coverageDefault, process.cwd(), log);
            }

            break;

          case 'jest':
            if (watchMode) {
              yield runShellCommand(packerConfig.test.advanced.jest.coverageWatch, process.cwd(), log);
            } else {
              yield runShellCommand(packerConfig.test.advanced.jest.coverageDefault, process.cwd(), log);
            }

            break;
        }
      } else {
        switch (packerConfig.test.framework) {
          case 'jasmine':
            if (watchMode) {
              yield watchSource(watchGlob,
              /*#__PURE__*/
              _asyncToGenerator(function* () {
                yield runShellCommand(packerConfig.test.advanced.jasmine.watch, process.cwd(), log);
              }));
            } else {
              yield runShellCommand(packerConfig.test.advanced.jasmine.default, process.cwd(), log);
            }

            break;

          case 'mocha':
            if (watchMode) {
              yield runShellCommand(packerConfig.test.advanced.mocha.watch, process.cwd(), log);
            } else {
              yield runShellCommand(packerConfig.test.advanced.mocha.default, process.cwd(), log);
            }

            break;

          case 'jest':
            if (watchMode) {
              yield runShellCommand(packerConfig.test.advanced.jest.watch, process.cwd(), log);
            } else {
              yield runShellCommand(packerConfig.test.advanced.jest.default, process.cwd(), log);
            }

            break;
        }
      }
    } catch (e) {
      log.error('test execution failure\n%o', e);
    }
  });

  return function runNodeUnitTest(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Get karma packer plugin
 */

var karmaPackerPlugin = log => {
  try {
    var typescript = requireDependency('typescript', log);
    var packerConfig = meta.readPackerConfig(log);
    var packageConfig = meta.readPackageData();
    var babelConfig = meta.readBabelConfig();
    var scriptExt = parseKarmaScriptPreprocessorExtensionGlob(packerConfig.compiler.script.preprocessor);
    var testGlob = "spec/**/*.spec.".concat(scriptExt);
    log.trace('test glob: %s', testGlob);
    var preprocessors = {};
    preprocessors[testGlob] = ['rollup'];
    var coveragePlugins = [];

    if (args.includes('--coverage') || args.includes('-C')) {
      log.trace('identified as coverage task');
      coveragePlugins = [rollupIstanbul({
        exclude: ['node_modules/**']
      })];
    }
    /**
     * This is just a normal Rollup config object,
     * except that `input` is handled for you.
     */


    var externals = extractBundleExternals(packerConfig);
    var rollupPreprocessor = {
      external: externals,
      output: {
        format: 'iife',
        name: 'test',
        sourcemap: 'inline',
        globals: packerConfig.bundle.globals
      },
      plugins: [...getStyleBuildPlugins(packerConfig, packageConfig, false, false, log), ...getPreBundlePlugins(packerConfig), ...getDependencyResolvePlugins(packerConfig), ...getScriptBuildPlugin('bundle', false, false, packerConfig, babelConfig, typescript, log), ...customRollupPlugins(packerConfig, 'bundle'), ...coveragePlugins]
    };
    var framework = String(packerConfig.test.framework).toLowerCase();
    return {
      rollupPreprocessor,
      preprocessors,
      karmaRollupPreprocessor,
      framework,
      testGlob
    };
  } catch (e) {
    log.error('task failure: %s\n', e.stack || e.message);
    process.exit(1);
  }
};

/**
 * Initialize test associated gulp tasks.
 */

function init$5() {
  /**
   * Unit test execution gulp task.
   *
   */
  gulp.task('test',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[test]');

    try {
      log.trace('start');
      var packerConfig = meta.readPackerConfig(log);
      /**
       * Use karma if browser build mode and not using jest, else use standalone test execution
       * depending on test framework.
       */

      if (packerConfig.test.environment === 'browser') {
        log.trace('start test suite execution via karma');
        var {
          Server,
          config
        } = requireDependency('karma', log);
        var karmaPackerConfig = {
          packer: karmaPackerPlugin(log)
        };
        var karmaConfig = config.parseConfig(path__default.join(process.cwd(), 'karma.conf.js'), karmaPackerConfig);
        var server = new Server(karmaConfig, exitCode => {
          log.info('Karma has exited with %d', exitCode);
          process.exit(exitCode);
        });
        server.start();
      } else {
        log.trace('start standalone test suite execution');
        yield runNodeUnitTest(packerConfig, log);
      }
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  }));
}

/**
 * Initialize source watch associated gulp tasks.
 */

function init$6() {
  /**
   * Watch and build source gulp task.
   * Watch source and realtime compile when source files change.
   * Serve and live reload demo page if bundle is browser compliant.
   */
  gulp.task('build:watch',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[watch]');

    try {
      var typescript = requireDependency('typescript', log);
      var packerConfig = meta.readPackerConfig(log);
      var packageConfig = meta.readPackageData();
      var babelConfig = meta.readBabelConfig();
      var banner = yield getBanner(packerConfig, packageConfig);
      var baseConfig = getBaseConfig(packerConfig, packageConfig, banner, 'inline');
      makeRelativeDirPath(packerConfig.tmp, 'watch');
      var rollupServePlugins = [];

      if (packerConfig.serve && packerConfig.bundle.format !== 'cjs') {
        log.trace('build bundle with serve support');
        var additionalServeDir = packerConfig.serve.serveDir.map(dir => path__default.join(process.cwd(), dir));
        rollupServePlugins = [rollupBrowserSync(mergeDeep({
          server: [path__default.join(process.cwd(), packerConfig.tmp, 'watch'), path__default.join(process.cwd(), packerConfig.serve.demoDir), path__default.join(process.cwd(), packerConfig.serve.helperDir), ...additionalServeDir],
          open: packerConfig.serve.open,
          ui: {
            port: packerConfig.serve.port
          },
          files: [path__default.join(process.cwd(), packerConfig.tmp, 'watch/**/*'), path__default.join(process.cwd(), packerConfig.serve.demoDir, '**/*')]
        }, packerConfig.compiler.advanced.rollup.pluginOptions.browserSync))];
      } else {
        log.trace('build serve disabled or not supported for bundle type');
      }

      var externals = extractBundleExternals(packerConfig);
      var targetFileName = "".concat(packageConfig.name, ".").concat(packerConfig.bundle.format, ".js");
      var watchConfig = merge({}, baseConfig, {
        external: externals,
        output: {
          file: path__default.join(process.cwd(), packerConfig.tmp, 'watch', targetFileName),
          format: packerConfig.bundle.format,
          globals: packerConfig.bundle.globals,
          name: packerConfig.bundle.namespace
        },
        plugins: [...getStyleBuildPlugins(packerConfig, packageConfig, true, true, log), ...getPreBundlePlugins(packerConfig), ...getDependencyResolvePlugins(packerConfig), ...getScriptBuildPlugin('bundle', false, false, packerConfig, babelConfig, typescript, log), ...rollupServePlugins, ...customRollupPlugins(packerConfig, 'bundle')],
        watch: {
          exclude: ['node_modules/**']
        }
      });
      log.trace('rollup config:\n%o', watchConfig);
      var watcher = yield rollup.watch([mergeDeep(watchConfig, packerConfig.compiler.advanced.rollup.watchOptions)]);
      watcher.on('event', event => {
        switch (event.code) {
          case 'START':
            log.info('%s - %s', 'watch', 'bundling start');
            break;

          case 'END':
            log.info('%s - %s', 'watch', 'bundling end');
            break;

          case 'ERROR':
            log.error('%s - %s\n%o', 'watch', 'bundling failure', event.error);
            break;

          case 'FATAL':
            log.error('%s - %s\n%o', 'watch', 'bundling crashed', event);
            break;
        }
      });
    } catch (e) {
      log.error('task failure: %s\n', e.stack || e.message);
      process.exit(1);
    }
  }));
  /**
   * Watch source gulp task.
   * Clean watch temporary files and start watch build task in series.
   */

  gulp.task('watch', gulp.series('watch:tmp:clean', 'build:watch'));
}

/**
 * Initialize version log associated gulp tasks.
 */

function init$7() {
  /**
   * Log packer version gulp task.
   */
  gulp.task('version',
  /*#__PURE__*/
  _asyncToGenerator(function* () {
    var log = Logger.create('[version]');

    try {
      console.log(chalk.blue('%s version: %s'), name, chalk.red(version));
    } catch (e) {
      log.error('task failure:\n', e.stack || e.message);
      process.exit(1);
    }
  }));
}

/**
 * Initialize packer CLI tasks.
 */

var initPacker =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* () {
    var log = Logger.create('');

    try {
      switch (args[0]) {
        case '--help':
        case '-h':
          console.log((yield meta.readPackerHelpSummary()));
          break;

        case '--version':
        case '-v':
          init$7();
          gulp.series('version')(() => {// no implementation
          });
          break;

        case 'generate':
        case 'g':
          init$2();
          console.log(chalk.red((yield meta.readPackerBanner())));
          gulp.series('generate')(() => {// no implementation
          });
          break;

        case 'build':
        case 'b':
          init();
          init$1();
          meta.fetchPackerConfig(log);
          gulp.series('build')(() => {// no implementation
          });
          break;

        case 'watch':
        case 'w':
          init();
          init$6();
          meta.fetchPackerConfig(log);
          gulp.series('watch')(() => {// no implementation
          });
          break;

        case 'test':
        case 't':
          init$5();
          meta.fetchPackerConfig(log);
          gulp.series('test')(() => {// no implementation
          });
          break;

        case 'clean':
        case 'c':
          init();
          meta.fetchPackerConfig(log);
          gulp.series('clean')(() => {// no implementation
          });
          break;

        case 'lint':
        case 'l':
          {
            init$3();
            meta.fetchPackerConfig(log);

            if (args.includes('--style') || args.includes('-st')) {
              gulp.series('lint:style')(() => {// no implementation
              });
              break;
            } else if (args.includes('--script') || args.includes('-sc')) {
              gulp.series('lint:script')(() => {// no implementation
              });
              break;
            } else {
              gulp.series('lint')(() => {// no implementation
              });
            }

            break;
          }

        case 'format':
        case 'f':
          {
            init$4();
            meta.fetchPackerConfig(log);
            gulp.series('format')(() => {// no implementation
            });
            break;
          }

        default:
          log.error('Invalid task name argument\n%s%s', chalk.reset('try '), chalk.blue('packer --help'));
          process.exit(1);
      }
    } catch (e) {
      log.error('packer init failure: %s\n', e.stack || e.message);
    }
  });

  return function initPacker() {
    return _ref.apply(this, arguments);
  };
}();

exports.initPacker = initPacker;
//# sourceMappingURL=packer-cli.cjs.js.map
