import { ScriptPreprocessor } from '../model/script-preprocessor';
import { StylePreprocessor } from '../model/style-preprocessor';
import { PackerOptions } from '../model/packer-options';
import { BuildMode } from '../model/build-mode';
import { TestEnvironment } from '../model/test-environment';
/**
 * Parse style extension by preprocessor type.
 * @param preprocessor Style preprocessor type.
 */
export declare const parseStylePreprocessorExtension: (preprocessor: StylePreprocessor) => string;
/**
 * Parse license file name by license type.
 * @param license License type.
 */
export declare const parseLicenseType: (license: string) => string;
/**
 * Parse script transpiler by preprocessor type.
 * @param preprocessor Script preprocessor type.
 */
export declare const parseScriptPreprocessorExtension: (preprocessor: ScriptPreprocessor) => string;
/**
 * Parse script transpiler by preprocessor type and extract extension glob.
 * @param preprocessor Script preprocessor type.
 */
export declare const parseScriptPreprocessorExtensionGlob: (preprocessor: ScriptPreprocessor) => string;
/**
 * Parse karma script transpiler by preprocessor type and extract extension glob.
 * @param preprocessor Script preprocessor type.
 */
export declare const parseKarmaScriptPreprocessorExtensionGlob: (preprocessor: ScriptPreprocessor) => string;
/**
 * Parse package build mode.
 * @param packerOptions Packer options object.
 */
export declare const parseBuildMode: (packerOptions: PackerOptions) => BuildMode;
/**
 * Parse test environment type.
 * @param packerOptions Packer options object.
 */
export declare const parseTestEnvironment: (packerOptions: PackerOptions) => TestEnvironment;
